@@ -6,7 +6,15 @@ use agent_client_protocol_schema::{
     LoadSessionResponse, NewSessionRequest, NewSessionResponse, PromptRequest, PromptResponse,
     Result, SetSessionModeRequest, SetSessionModeResponse,
 };
-#[cfg(feature = "unstable")]
+#[cfg(feature = "unstable_session_fork")]
+use agent_client_protocol_schema::{ForkSessionRequest, ForkSessionResponse};
+#[cfg(feature = "unstable_session_list")]
+use agent_client_protocol_schema::{ListSessionsRequest, ListSessionsResponse};
+#[cfg(feature = "unstable_session_resume")]
+use agent_client_protocol_schema::{ResumeSessionRequest, ResumeSessionResponse};
+#[cfg(feature = "unstable_session_config_options")]
+use agent_client_protocol_schema::{SetSessionConfigOptionRequest, SetSessionConfigOptionResponse};
+#[cfg(feature = "unstable_session_model")]
 use agent_client_protocol_schema::{SetSessionModelRequest, SetSessionModelResponse};
 use serde_json::value::RawValue;
 
@@ -118,7 +126,7 @@ pub trait Agent {
     /// This capability is not part of the spec yet, and may be removed or changed at any point.
     ///
     /// Select a model for a given session.
-    #[cfg(feature = "unstable")]
+    #[cfg(feature = "unstable_session_model")]
     async fn set_session_model(
         &self,
         _args: SetSessionModelRequest,
@@ -126,6 +134,64 @@ pub trait Agent {
         Err(Error::method_not_found())
     }
 
+    /// **UNSTABLE**
+    ///
+    /// This capability is not part of the spec yet, and may be removed or changed at any point.
+    ///
+    /// Sets the current value for a session configuration option.
+    ///
+    /// Configuration options allow agents to expose arbitrary selectors (like model choice,
+    /// reasoning level, etc.) that clients can display and modify.
+    ///
+    /// The response returns the full list of configuration options with their current values,
+    /// as changing one option may affect others.
+    #[cfg(feature = "unstable_session_config_options")]
+    async fn set_session_config_option(
+        &self,
+        _args: SetSessionConfigOptionRequest,
+    ) -> Result<SetSessionConfigOptionResponse> {
+        Err(Error::method_not_found())
+    }
+
+    /// **UNSTABLE**
+    ///
+    /// This capability is not part of the spec yet, and may be removed or changed at any point.
+    ///
+    /// Lists existing sessions known to the agent.
+    ///
+    /// Only available if the Agent supports the `sessionCapabilities.list` capability.
+    #[cfg(feature = "unstable_session_list")]
+    async fn list_sessions(&self, _args: ListSessionsRequest) -> Result<ListSessionsResponse> {
+        Err(Error::method_not_found())
+    }
+
+    /// **UNSTABLE**
+    ///
+    /// This capability is not part of the spec yet, and may be removed or changed at any point.
+    ///
+    /// Forks an existing session, creating a new session with the same conversation history.
+    ///
+    /// Only available if the Agent supports the `sessionCapabilities.fork` capability.
+    #[cfg(feature = "unstable_session_fork")]
+    async fn fork_session(&self, _args: ForkSessionRequest) -> Result<ForkSessionResponse> {
+        Err(Error::method_not_found())
+    }
+
+    /// **UNSTABLE**
+    ///
+    /// This capability is not part of the spec yet, and may be removed or changed at any point.
+    ///
+    /// Resumes an existing session without replaying message history.
+    ///
+    /// This is similar to `load_session`, except it does not return previous messages.
+    /// Useful for agents that support continuing conversations but don't store full history.
+    ///
+    /// Only available if the Agent supports the `sessionCapabilities.resume` capability.
+    #[cfg(feature = "unstable_session_resume")]
+    async fn resume_session(&self, _args: ResumeSessionRequest) -> Result<ResumeSessionResponse> {
+        Err(Error::method_not_found())
+    }
+
     /// Handles extension method requests from the client.
     ///
     /// Extension methods provide a way to add custom functionality while maintaining
@@ -173,13 +239,32 @@ impl<T: Agent> Agent for Rc<T> {
     async fn cancel(&self, args: CancelNotification) -> Result<()> {
         self.as_ref().cancel(args).await
     }
-    #[cfg(feature = "unstable")]
+    #[cfg(feature = "unstable_session_model")]
     async fn set_session_model(
         &self,
         args: SetSessionModelRequest,
     ) -> Result<SetSessionModelResponse> {
         self.as_ref().set_session_model(args).await
     }
+    #[cfg(feature = "unstable_session_config_options")]
+    async fn set_session_config_option(
+        &self,
+        args: SetSessionConfigOptionRequest,
+    ) -> Result<SetSessionConfigOptionResponse> {
+        self.as_ref().set_session_config_option(args).await
+    }
+    #[cfg(feature = "unstable_session_list")]
+    async fn list_sessions(&self, args: ListSessionsRequest) -> Result<ListSessionsResponse> {
+        self.as_ref().list_sessions(args).await
+    }
+    #[cfg(feature = "unstable_session_fork")]
+    async fn fork_session(&self, args: ForkSessionRequest) -> Result<ForkSessionResponse> {
+        self.as_ref().fork_session(args).await
+    }
+    #[cfg(feature = "unstable_session_resume")]
+    async fn resume_session(&self, args: ResumeSessionRequest) -> Result<ResumeSessionResponse> {
+        self.as_ref().resume_session(args).await
+    }
     async fn ext_method(&self, args: ExtRequest) -> Result<ExtResponse> {
         self.as_ref().ext_method(args).await
     }
@@ -214,13 +299,32 @@ impl<T: Agent> Agent for Arc<T> {
     async fn cancel(&self, args: CancelNotification) -> Result<()> {
         self.as_ref().cancel(args).await
     }
-    #[cfg(feature = "unstable")]
+    #[cfg(feature = "unstable_session_model")]
     async fn set_session_model(
         &self,
         args: SetSessionModelRequest,
     ) -> Result<SetSessionModelResponse> {
         self.as_ref().set_session_model(args).await
     }
+    #[cfg(feature = "unstable_session_config_options")]
+    async fn set_session_config_option(
+        &self,
+        args: SetSessionConfigOptionRequest,
+    ) -> Result<SetSessionConfigOptionResponse> {
+        self.as_ref().set_session_config_option(args).await
+    }
+    #[cfg(feature = "unstable_session_list")]
+    async fn list_sessions(&self, args: ListSessionsRequest) -> Result<ListSessionsResponse> {
+        self.as_ref().list_sessions(args).await
+    }
+    #[cfg(feature = "unstable_session_fork")]
+    async fn fork_session(&self, args: ForkSessionRequest) -> Result<ForkSessionResponse> {
+        self.as_ref().fork_session(args).await
+    }
+    #[cfg(feature = "unstable_session_resume")]
+    async fn resume_session(&self, args: ResumeSessionRequest) -> Result<ResumeSessionResponse> {
+        self.as_ref().resume_session(args).await
+    }
     async fn ext_method(&self, args: ExtRequest) -> Result<ExtResponse> {
         self.as_ref().ext_method(args).await
     }
