@@ -133,7 +133,7 @@ impl Client for TestClient {
 
 #[derive(Clone)]
 struct TestAgent {
-    sessions: Arc<Mutex<std::collections::HashSet<SessionId>>>,
+    sessions: Arc<Mutex<std::collections::HashMap<SessionId, std::path::PathBuf>>>,
     prompts_received: Arc<Mutex<Vec<PromptReceived>>>,
     cancellations_received: Arc<Mutex<Vec<SessionId>>>,
     extension_notifications: Arc<Mutex<Vec<(String, ExtNotification)>>>,
@@ -144,7 +144,7 @@ type PromptReceived = (SessionId, Vec<ContentBlock>);
 impl TestAgent {
     fn new() -> Self {
         Self {
-            sessions: Arc::new(Mutex::new(std::collections::HashSet::new())),
+            sessions: Arc::new(Mutex::new(std::collections::HashMap::new())),
             prompts_received: Arc::new(Mutex::new(Vec::new())),
             cancellations_received: Arc::new(Mutex::new(Vec::new())),
             extension_notifications: Arc::new(Mutex::new(Vec::new())),
@@ -163,9 +163,12 @@ impl Agent for TestAgent {
         Ok(AuthenticateResponse::default())
     }
 
-    async fn new_session(&self, _arguments: NewSessionRequest) -> Result<NewSessionResponse> {
+    async fn new_session(&self, arguments: NewSessionRequest) -> Result<NewSessionResponse> {
         let session_id = SessionId::new("test-session-123");
-        self.sessions.lock().unwrap().insert(session_id.clone());
+        self.sessions
+            .lock()
+            .unwrap()
+            .insert(session_id.clone(), arguments.cwd);
         Ok(NewSessionResponse::new(session_id))
     }
 
@@ -196,7 +199,7 @@ impl Agent for TestAgent {
         Ok(())
     }
 
-    #[cfg(feature = "unstable")]
+    #[cfg(feature = "unstable_session_model")]
     async fn set_session_model(
         &self,
         args: agent_client_protocol_schema::SetSessionModelRequest,
@@ -205,6 +208,74 @@ impl Agent for TestAgent {
         Ok(agent_client_protocol_schema::SetSessionModelResponse::default())
     }
 
+    #[cfg(feature = "unstable_session_list")]
+    async fn list_sessions(
+        &self,
+        _args: agent_client_protocol_schema::ListSessionsRequest,
+    ) -> Result<agent_client_protocol_schema::ListSessionsResponse> {
+        let sessions = self.sessions.lock().unwrap();
+        let session_infos: Vec<_> = sessions
+            .iter()
+            .map(|(id, cwd)| {
+                agent_client_protocol_schema::SessionInfo::new(id.clone(), cwd.clone())
+            })
+            .collect();
+        Ok(agent_client_protocol_schema::ListSessionsResponse::new(
+            session_infos,
+        ))
+    }
+
+    #[cfg(feature = "unstable_session_fork")]
+    async fn fork_session(
+        &self,
+        args: agent_client_protocol_schema::ForkSessionRequest,
+    ) -> Result<agent_client_protocol_schema::ForkSessionResponse> {
+        let new_session_id = SessionId::new(format!("fork-of-{}", args.session_id.0));
+        self.sessions
+            .lock()
+            .unwrap()
+            .insert(new_session_id.clone(), args.cwd);
+        Ok(agent_client_protocol_schema::ForkSessionResponse::new(
+            new_session_id,
+        ))
+    }
+
+    #[cfg(feature = "unstable_session_resume")]
+    async fn resume_session(
+        &self,
+        args: agent_client_protocol_schema::ResumeSessionRequest,
+    ) -> Result<agent_client_protocol_schema::ResumeSessionResponse> {
+        // Check if session exists
+        if !self.sessions.lock().unwrap().contains_key(&args.session_id) {
+            return Err(Error::invalid_params());
+        }
+        Ok(agent_client_protocol_schema::ResumeSessionResponse::new())
+    }
+
+    #[cfg(feature = "unstable_session_config_options")]
+    async fn set_session_config_option(
+        &self,
+        args: agent_client_protocol_schema::SetSessionConfigOptionRequest,
+    ) -> Result<agent_client_protocol_schema::SetSessionConfigOptionResponse> {
+        Ok(
+            agent_client_protocol_schema::SetSessionConfigOptionResponse::new(vec![
+                agent_client_protocol_schema::SessionConfigOption::select(
+                    args.config_id,
+                    "Test Option",
+                    args.value,
+                    vec![
+                        agent_client_protocol_schema::SessionConfigSelectOption::new(
+                            "value1", "Value 1",
+                        ),
+                        agent_client_protocol_schema::SessionConfigSelectOption::new(
+                            "value2", "Value 2",
+                        ),
+                    ],
+                ),
+            ]),
+        )
+    }
+
     async fn ext_method(&self, args: ExtRequest) -> Result<ExtResponse> {
         dbg!();
         match dbg!(args.method.as_ref()) {
@@ -655,3 +726,206 @@ async fn test_extension_methods_and_notifications() {
         })
         .await;
 }
+
+#[cfg(feature = "unstable_session_fork")]
+#[tokio::test]
+async fn test_fork_session() {
+    let local_set = tokio::task::LocalSet::new();
+    local_set
+        .run_until(async {
+            let client = TestClient::new();
+            let agent = TestAgent::new();
+
+            let (agent_conn, _client_conn) = create_connection_pair(&client, &agent);
+
+            // First create a session
+            let new_session_response = agent_conn
+                .new_session(NewSessionRequest::new("/test"))
+                .await
+                .expect("new_session failed");
+
+            let original_session_id = new_session_response.session_id;
+
+            // Fork the session
+            let fork_response = agent_conn
+                .fork_session(agent_client_protocol_schema::ForkSessionRequest::new(
+                    original_session_id.clone(),
+                    "/test",
+                ))
+                .await
+                .expect("fork_session failed");
+
+            // Verify the forked session has a different ID
+            assert_ne!(fork_response.session_id, original_session_id);
+            assert_eq!(
+                fork_response.session_id.0.as_ref(),
+                format!("fork-of-{}", original_session_id.0)
+            );
+
+            // Verify the forked session was added to the agent's sessions
+            let sessions = agent.sessions.lock().unwrap();
+            assert!(sessions.contains_key(&fork_response.session_id));
+        })
+        .await;
+}
+
+#[cfg(feature = "unstable_session_list")]
+#[tokio::test]
+async fn test_list_sessions() {
+    let local_set = tokio::task::LocalSet::new();
+    local_set
+        .run_until(async {
+            let client = TestClient::new();
+            let agent = TestAgent::new();
+
+            let (agent_conn, _client_conn) = create_connection_pair(&client, &agent);
+
+            // First create a session
+            let new_session_response = agent_conn
+                .new_session(NewSessionRequest::new("/test"))
+                .await
+                .expect("new_session failed");
+
+            // Verify the session was created
+            assert!(!new_session_response.session_id.0.is_empty());
+
+            // List sessions
+            let list_response = agent_conn
+                .list_sessions(agent_client_protocol_schema::ListSessionsRequest::new())
+                .await
+                .expect("list_sessions failed");
+
+            // Verify the response contains our session
+            assert_eq!(list_response.sessions.len(), 1);
+            assert_eq!(
+                list_response.sessions[0].session_id,
+                new_session_response.session_id
+            );
+            assert_eq!(
+                list_response.sessions[0].cwd,
+                std::path::PathBuf::from("/test")
+            );
+            assert!(list_response.next_cursor.is_none());
+        })
+        .await;
+}
+
+#[cfg(feature = "unstable_session_resume")]
+#[tokio::test]
+async fn test_resume_session() {
+    let local_set = tokio::task::LocalSet::new();
+    local_set
+        .run_until(async {
+            let client = TestClient::new();
+            let agent = TestAgent::new();
+
+            let (agent_conn, _client_conn) = create_connection_pair(&client, &agent);
+
+            // First create a session
+            let new_session_response = agent_conn
+                .new_session(NewSessionRequest::new("/test"))
+                .await
+                .expect("new_session failed");
+
+            let session_id = new_session_response.session_id;
+
+            // Resume the session
+            let resume_response = agent_conn
+                .resume_session(agent_client_protocol_schema::ResumeSessionRequest::new(
+                    session_id.clone(),
+                    "/test",
+                ))
+                .await
+                .expect("resume_session failed");
+
+            // Verify we got a valid response (no modes by default in TestAgent)
+            assert!(resume_response.modes.is_none());
+        })
+        .await;
+}
+
+#[cfg(feature = "unstable_session_info_update")]
+#[tokio::test]
+async fn test_session_info_update() {
+    let local_set = tokio::task::LocalSet::new();
+    local_set
+        .run_until(async {
+            let client = TestClient::new();
+            let agent = TestAgent::new();
+
+            let (_agent_conn, client_conn) = create_connection_pair(&client, &agent);
+
+            let session_id = SessionId::new("test-session");
+
+            // Send a session info update notification
+            client_conn
+                .session_notification(SessionNotification::new(
+                    session_id.clone(),
+                    SessionUpdate::SessionInfoUpdate(
+                        agent_client_protocol_schema::SessionInfoUpdate::new()
+                            .title("Test Session Title")
+                            .updated_at("2025-01-15T12:00:00Z"),
+                    ),
+                ))
+                .await
+                .expect("session_notification failed");
+
+            tokio::task::yield_now().await;
+
+            // Verify client received the notification
+            let notifications = client.session_notifications.lock().unwrap();
+            assert_eq!(notifications.len(), 1);
+            assert_eq!(notifications[0].session_id, session_id);
+
+            if let SessionUpdate::SessionInfoUpdate(info_update) = &notifications[0].update {
+                assert_eq!(
+                    info_update.title,
+                    agent_client_protocol_schema::MaybeUndefined::Value(
+                        "Test Session Title".to_string()
+                    )
+                );
+                assert_eq!(
+                    info_update.updated_at,
+                    agent_client_protocol_schema::MaybeUndefined::Value(
+                        "2025-01-15T12:00:00Z".to_string()
+                    )
+                );
+            } else {
+                panic!("Expected SessionInfoUpdate variant");
+            }
+        })
+        .await;
+}
+
+#[cfg(feature = "unstable_session_config_options")]
+#[tokio::test]
+async fn test_set_session_config_option() {
+    let local_set = tokio::task::LocalSet::new();
+    local_set
+        .run_until(async {
+            let client = TestClient::new();
+            let agent = TestAgent::new();
+
+            let (agent_conn, _client_conn) = create_connection_pair(&client, &agent);
+
+            // Set a config option
+            let response = agent_conn
+                .set_session_config_option(
+                    agent_client_protocol_schema::SetSessionConfigOptionRequest::new(
+                        "test-session",
+                        "mode",
+                        "value2",
+                    ),
+                )
+                .await
+                .expect("set_session_config_option failed");
+
+            // Verify we got config options back
+            assert_eq!(response.config_options.len(), 1);
+            assert_eq!(
+                response.config_options[0].id,
+                agent_client_protocol_schema::SessionConfigId::new("mode")
+            );
+        })
+        .await;
+}
