@@ -4,8 +4,7 @@ use crate::ir::pcc::*;
 use crate::ir::types::*;
 use crate::isa::x64::args::AvxOpcode;
 use crate::isa::x64::inst::args::{
-    AluRmiROpcode, Amode, Gpr, Imm8Reg, RegMem, RegMemImm, ShiftKind, SseOpcode, SyntheticAmode,
-    ToWritableReg, CC,
+    Amode, Gpr, Imm8Reg, RegMem, RegMemImm, ShiftKind, SseOpcode, SyntheticAmode, ToWritableReg, CC,
 };
 use crate::isa::x64::inst::Inst;
 use crate::machinst::pcc::*;
@@ -63,119 +62,13 @@ pub(crate) fn check(
             Ok(())
         }
 
-        Inst::AluRmiR {
-            size,
-            op: AluRmiROpcode::Add,
-            src1,
-            ref src2,
-            dst,
-        } => match *<&RegMemImm>::from(src2) {
-            RegMemImm::Reg { reg: src2 } => {
-                let bits = size.to_bits().into();
-                check_binop(
-                    ctx,
-                    vcode,
-                    64,
-                    dst.to_writable_reg(),
-                    src1.to_reg(),
-                    src2,
-                    |src1, src2| clamp_range(ctx, 64, bits, ctx.add(src1, src2, bits)),
-                )
-            }
-            RegMemImm::Imm { simm32 } => {
-                let bits = size.to_bits().into();
-                check_unop(
-                    ctx,
-                    vcode,
-                    64,
-                    dst.to_writable_reg(),
-                    src1.to_reg(),
-                    |src1| {
-                        let simm32: i64 = simm32.into();
-                        clamp_range(ctx, 64, bits, ctx.offset(src1, bits, simm32))
-                    },
-                )
-            }
-            RegMemImm::Mem { ref addr } => {
-                let bits: u16 = size.to_bits().into();
-                let loaded = check_load(ctx, None, addr, vcode, size.to_type(), bits)?;
-                check_unop(ctx, vcode, 64, dst.to_writable_reg(), src1.into(), |src1| {
-                    let sum = loaded.and_then(|loaded| ctx.add(src1, &loaded, bits));
-                    clamp_range(ctx, 64, bits, sum)
-                })
-            }
-        },
-
-        Inst::AluRmiR {
-            size,
-            op: AluRmiROpcode::Sub,
-            src1,
-            ref src2,
-            dst,
-        } => match *<&RegMemImm>::from(src2) {
-            RegMemImm::Imm { simm32 } => {
-                let bits = size.to_bits().into();
-                check_unop(
-                    ctx,
-                    vcode,
-                    64,
-                    dst.to_writable_reg(),
-                    src1.to_reg(),
-                    |src1| {
-                        let simm32: i64 = simm32.into();
-                        clamp_range(ctx, 64, bits, ctx.offset(src1, bits, -simm32))
-                    },
-                )
-            }
-            RegMemImm::Reg { .. } => {
-                let bits: u16 = size.to_bits().into();
-                check_output(ctx, vcode, dst.to_writable_reg(), &[], |_vcode| {
-                    clamp_range(ctx, 64, bits, None)
-                })
-            }
-            RegMemImm::Mem { ref addr } => {
-                let loaded = check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
-                check_output(ctx, vcode, dst.to_writable_reg(), &[], |_vcode| {
-                    clamp_range(ctx, 64, size.to_bits().into(), loaded)
-                })
-            }
-        },
-
-        Inst::AluRmiR {
-            size,
-            ref src2,
-            dst,
-            ..
-        } => match <&RegMemImm>::from(src2) {
-            RegMemImm::Mem { ref addr } => {
-                let loaded = check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
-                check_output(ctx, vcode, dst.to_writable_reg(), &[], |_vcode| {
-                    clamp_range(ctx, 64, size.to_bits().into(), loaded)
-                })
-            }
-            RegMemImm::Reg { .. } | RegMemImm::Imm { .. } => {
-                undefined_result(ctx, vcode, dst, 64, size.to_bits().into())
-            }
-        },
-
-        Inst::AluRM {
-            size,
-            op: _,
-            ref src1_dst,
-            src2: _,
-            lock: _,
-        } => {
-            check_load(ctx, None, src1_dst, vcode, size.to_type(), 64)?;
-            check_store(ctx, None, src1_dst, vcode, size.to_type())
-        }
-
         Inst::AluRmRVex {
             size,
             ref src2,
             dst,
             ..
         } => match <&RegMem>::from(src2) {
-            RegMem::Mem { ref addr } => {
+            RegMem::Mem { addr } => {
                 let loaded = check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
                 check_output(ctx, vcode, dst.to_writable_reg(), &[], |_vcode| {
                     clamp_range(ctx, 64, size.to_bits().into(), loaded)
@@ -184,16 +77,6 @@ pub(crate) fn check(
             RegMem::Reg { .. } => undefined_result(ctx, vcode, dst, 64, size.to_bits().into()),
         },
 
-        Inst::AluConstOp {
-            op: AluRmiROpcode::Xor,
-            dst,
-            ..
-        } => check_output(ctx, vcode, dst.to_writable_reg(), &[], |_vcode| {
-            Ok(Some(Fact::constant(64, 0)))
-        }),
-
-        Inst::AluConstOp { dst, .. } => undefined_result(ctx, vcode, dst, 64, 64),
-
         Inst::UnaryRmR {
             size, ref src, dst, ..
         }
@@ -203,7 +86,7 @@ pub(crate) fn check(
         | Inst::UnaryRmRImmVex {
             size, ref src, dst, ..
         } => match <&RegMem>::from(src) {
-            RegMem::Mem { ref addr } => {
+            RegMem::Mem { addr } => {
                 check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
                 check_output(ctx, vcode, dst.to_writable_reg(), &[], |_vcode| {
                     clamp_range(ctx, 64, size.to_bits().into(), None)
@@ -224,7 +107,7 @@ pub(crate) fn check(
             ..
         } => {
             match <&RegMem>::from(divisor) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -237,7 +120,7 @@ pub(crate) fn check(
             dst, ref divisor, ..
         } => {
             match <&RegMem>::from(divisor) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8, 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -262,7 +145,7 @@ pub(crate) fn check(
             ..
         } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -273,7 +156,7 @@ pub(crate) fn check(
         }
         Inst::Mul8 { dst, ref src2, .. } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8, 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -288,7 +171,7 @@ pub(crate) fn check(
             ..
         } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -303,7 +186,7 @@ pub(crate) fn check(
             ..
         } => {
             match <&RegMem>::from(src1) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -351,7 +234,7 @@ pub(crate) fn check(
                         clamp_range(ctx, 64, from_bytes * 8, Some(src.clone()))
                     })
                 }
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     let loaded = check_load(
                         ctx,
                         Some(dst.to_writable_reg()),
@@ -397,7 +280,7 @@ pub(crate) fn check(
             dst,
         } => {
             match <&RegMem>::from(src) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, ext_mode.src_type(), 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -437,7 +320,7 @@ pub(crate) fn check(
 
         Inst::XmmRmiReg { dst, ref src2, .. } => {
             match <&RegMemImm>::from(src2) {
-                RegMemImm::Mem { ref addr } => {
+                RegMemImm::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8X16, 128)?;
                 }
                 _ => {}
@@ -465,7 +348,7 @@ pub(crate) fn check(
                 }
                 Ok(())
             }
-            RegMemImm::Mem { ref addr } => {
+            RegMemImm::Mem { addr } => {
                 if let Some(rhs) = check_load(ctx, None, addr, vcode, size.to_type(), 64)? {
                     let lhs = get_fact_or_default(vcode, src1.to_reg(), 64);
                     state.cmp_flags = Some((lhs, rhs));
@@ -498,7 +381,7 @@ pub(crate) fn check(
             cc,
             ..
         } => match <&RegMem>::from(consequent) {
-            RegMem::Mem { ref addr } => {
+            RegMem::Mem { addr } => {
                 check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
                 Ok(())
             }
@@ -533,7 +416,7 @@ pub(crate) fn check(
         Inst::XmmCmove { dst, .. } => ensure_no_fact(vcode, dst.to_writable_reg().to_reg()),
 
         Inst::Push64 { ref src } => match <&RegMemImm>::from(src) {
-            RegMemImm::Mem { ref addr } => {
+            RegMemImm::Mem { addr } => {
                 check_load(ctx, None, addr, vcode, I64, 64)?;
                 Ok(())
             }
@@ -553,7 +436,7 @@ pub(crate) fn check(
             dst, src: ref src2, ..
         } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8X16, 128)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -568,7 +451,7 @@ pub(crate) fn check(
             ..
         } => {
             match <&RegMem>::from(src) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, F32, 32)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -582,7 +465,7 @@ pub(crate) fn check(
             ..
         } => {
             match <&RegMem>::from(src) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, F64, 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -610,7 +493,7 @@ pub(crate) fn check(
             ..
         } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8X16, 128)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -655,7 +538,7 @@ pub(crate) fn check(
             };
 
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, ty, size)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -665,7 +548,7 @@ pub(crate) fn check(
 
         Inst::XmmRmiRVex { dst, ref src2, .. } => {
             match <&RegMemImm>::from(src2) {
-                RegMemImm::Mem { ref addr } => {
+                RegMemImm::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8X16, 128)?;
                 }
                 RegMemImm::Reg { .. } | RegMemImm::Imm { .. } => {}
@@ -675,7 +558,7 @@ pub(crate) fn check(
 
         Inst::XmmVexPinsr { dst, ref src2, .. } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I64, 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -691,7 +574,7 @@ pub(crate) fn check(
 
         Inst::GprToXmmVex { dst, ref src, .. } | Inst::GprToXmm { dst, ref src, .. } => {
             match <&RegMem>::from(src) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I64, 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -731,7 +614,7 @@ pub(crate) fn check(
         Inst::CvtIntToFloat { dst, ref src2, .. }
         | Inst::CvtIntToFloatVex { dst, ref src2, .. } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I64, 64)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -783,7 +666,7 @@ pub(crate) fn check(
             ref src1, ref src2, ..
         } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8X16, 128)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -799,7 +682,7 @@ pub(crate) fn check(
             ..
         } if op.has_scalar_src2() => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(
                         ctx,
                         None,
@@ -816,7 +699,7 @@ pub(crate) fn check(
 
         Inst::XmmRmRImm { dst, ref src2, .. } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, I8X16, 128)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -828,7 +711,7 @@ pub(crate) fn check(
             ref src1, ref src2, ..
         } => {
             match <&RegMem>::from(src2) {
-                RegMem::Mem { ref addr } => {
+                RegMem::Mem { addr } => {
                     check_load(ctx, None, addr, vcode, F32, 32)?;
                 }
                 RegMem::Reg { .. } => {}
@@ -840,8 +723,9 @@ pub(crate) fn check(
         | Inst::ReturnCallKnown { .. }
         | Inst::JmpKnown { .. }
         | Inst::Ret { .. }
-        | Inst::JmpIf { .. }
+        | Inst::WinchJmpIf { .. }
         | Inst::JmpCond { .. }
+        | Inst::JmpCondOr { .. }
         | Inst::TrapIf { .. }
         | Inst::TrapIfAnd { .. }
         | Inst::TrapIfOr { .. }
@@ -852,7 +736,7 @@ pub(crate) fn check(
         Inst::ReturnCallUnknown { .. } => Ok(()),
 
         Inst::CallUnknown { ref info } => match <&RegMem>::from(&info.dest) {
-            RegMem::Mem { ref addr } => {
+            RegMem::Mem { addr } => {
                 check_load(ctx, None, addr, vcode, I64, 64)?;
                 Ok(())
             }
@@ -861,7 +745,7 @@ pub(crate) fn check(
         Inst::JmpUnknown {
             target: ref dest, ..
         } => match <&RegMem>::from(dest) {
-            RegMem::Mem { ref addr } => {
+            RegMem::Mem { addr } => {
                 check_load(ctx, None, addr, vcode, I64, 64)?;
                 Ok(())
             }
@@ -967,6 +851,10 @@ pub(crate) fn check(
             ensure_no_fact(vcode, dst.to_writable_reg().to_reg())
         }
 
+        Inst::GprUninitializedValue { dst } => {
+            ensure_no_fact(vcode, dst.to_writable_reg().to_reg())
+        }
+
         Inst::ElfTlsGetAddr { dst, .. } | Inst::MachOTlsGetAddr { dst, .. } => {
             ensure_no_fact(vcode, dst.to_writable_reg().to_reg())
         }
@@ -979,6 +867,8 @@ pub(crate) fn check(
         Inst::Unwind { .. } | Inst::DummyUse { .. } => Ok(()),
 
         Inst::StackSwitchBasic { .. } => Err(PccError::UnimplementedInst),
+
+        Inst::External { .. } => Ok(()), // TODO: unsure what to do about this!
     }
 }
 
