@@ -18,13 +18,15 @@ mod writer;
 
 pub use spirv::{Capability, SourceLanguage};
 
-use crate::arena::{Handle, HandleVec};
-use crate::proc::{BoundsCheckPolicies, TypeResolution};
+use alloc::{string::String, vec::Vec};
+use core::ops;
 
 use spirv::Word;
-use std::ops;
 use thiserror::Error;
 
+use crate::arena::{Handle, HandleVec};
+use crate::proc::{BoundsCheckPolicies, TypeResolution};
+
 #[derive(Clone)]
 struct PhysicalLayout {
     magic_number: Word,
@@ -73,6 +75,8 @@ pub enum Error {
     Validation(&'static str),
     #[error("overrides should not be present at this stage")]
     Override,
+    #[error(transparent)]
+    ResolveArraySizeError(#[from] crate::proc::ResolveArraySizeError),
 }
 
 #[derive(Default)]
@@ -144,13 +148,20 @@ struct Function {
     signature: Option<Instruction>,
     parameters: Vec<FunctionArgument>,
     variables: crate::FastHashMap<Handle<crate::LocalVariable>, LocalVariable>,
+    /// List of local variables used as a counters to ensure that all loops are bounded.
+    force_loop_bounding_vars: Vec<LocalVariable>,
 
-    /// A map taking an expression that yields a composite value (array, matrix)
-    /// to the temporary variables we have spilled it to, if any. Spilling
-    /// allows us to render an arbitrary chain of [`Access`] and [`AccessIndex`]
-    /// expressions as an `OpAccessChain` and an `OpLoad` (plus bounds checks).
-    /// This supports dynamic indexing of by-value arrays and matrices, which
-    /// SPIR-V does not.
+    /// A map from a Naga expression to the temporary SPIR-V variable we have
+    /// spilled its value to, if any.
+    ///
+    /// Naga IR lets us apply [`Access`] expressions to expressions whose value
+    /// is an array or matrix---not a pointer to such---but SPIR-V doesn't have
+    /// instructions that can do the same. So when we encounter such code, we
+    /// spill the expression's value to a generated temporary variable. That, we
+    /// can obtain a pointer to, and then use an `OpAccessChain` instruction to
+    /// do whatever series of [`Access`] and [`AccessIndex`] operations we need
+    /// (with bounds checks). Finally, we generate an `OpLoad` to get the final
+    /// value.
     ///
     /// [`Access`]: crate::Expression::Access
     /// [`AccessIndex`]: crate::Expression::AccessIndex
@@ -302,6 +313,26 @@ impl NumericType {
             _ => None,
         }
     }
+
+    const fn scalar(self) -> crate::Scalar {
+        match self {
+            NumericType::Scalar(scalar)
+            | NumericType::Vector { scalar, .. }
+            | NumericType::Matrix { scalar, .. } => scalar,
+        }
+    }
+
+    const fn with_scalar(self, scalar: crate::Scalar) -> Self {
+        match self {
+            NumericType::Scalar(_) => NumericType::Scalar(scalar),
+            NumericType::Vector { size, .. } => NumericType::Vector { size, scalar },
+            NumericType::Matrix { columns, rows, .. } => NumericType::Matrix {
+                columns,
+                rows,
+                scalar,
+            },
+        }
+    }
 }
 
 /// A SPIR-V type constructed during code generation.
@@ -319,9 +350,9 @@ impl NumericType {
 /// never synthesizes new struct types, so `LocalType` has nothing for that.
 ///
 /// Each `LocalType` variant should be handled identically to its analogous
-/// `TypeInner` variant. You can use the [`LocalType::from_inner`] function to
-/// help with this, by converting everything possible to a `LocalType` before
-/// inspecting it.
+/// `TypeInner` variant. You can use the [`Writer::localtype_from_inner`]
+/// function to help with this, by converting everything possible to a
+/// `LocalType` before inspecting it.
 ///
 /// ## `LocalType` equality and SPIR-V `OpType` uniqueness
 ///
@@ -341,8 +372,10 @@ impl NumericType {
 /// variant, is designed to help us deduplicate `OpTypeImage` instructions. See
 /// its documentation for details.
 ///
-/// `LocalType` also includes variants like `Pointer` that do not need to be
-/// unique - but it is harmless to avoid the duplication.
+/// SPIR-V does not require pointer types to be unique - but different
+/// SPIR-V ids are considered to be distinct pointer types. Since Naga
+/// uses structural type equality, we need to represent each Naga
+/// equivalence class with a single SPIR-V `OpTypePointer`.
 ///
 /// As it always must, the `Hash` implementation respects the `Eq` relation.
 ///
@@ -351,12 +384,8 @@ impl NumericType {
 enum LocalType {
     /// A numeric type.
     Numeric(NumericType),
-    LocalPointer {
-        base: NumericType,
-        class: spirv::StorageClass,
-    },
     Pointer {
-        base: Handle<crate::Type>,
+        base: Word,
         class: spirv::StorageClass,
     },
     Image(LocalImageType),
@@ -364,16 +393,6 @@ enum LocalType {
         image_type_id: Word,
     },
     Sampler,
-    /// Equivalent to a [`LocalType::Pointer`] whose `base` is a Naga IR [`BindingArray`]. SPIR-V
-    /// permits duplicated `OpTypePointer` ids, so it's fine to have two different [`LocalType`]
-    /// representations for pointer types.
-    ///
-    /// [`BindingArray`]: crate::TypeInner::BindingArray
-    PointerToBindingArray {
-        base: Handle<crate::Type>,
-        size: u32,
-        space: crate::AddressSpace,
-    },
     BindingArray {
         base: Handle<crate::Type>,
         size: u32,
@@ -422,52 +441,6 @@ struct LookupFunctionType {
     return_type_id: Word,
 }
 
-impl LocalType {
-    fn from_inner(inner: &crate::TypeInner) -> Option<Self> {
-        Some(match *inner {
-            crate::TypeInner::Scalar(_)
-            | crate::TypeInner::Atomic(_)
-            | crate::TypeInner::Vector { .. }
-            | crate::TypeInner::Matrix { .. } => {
-                // We expect `NumericType::from_inner` to handle all
-                // these cases, so unwrap.
-                LocalType::Numeric(NumericType::from_inner(inner).unwrap())
-            }
-            crate::TypeInner::Pointer { base, space } => LocalType::Pointer {
-                base,
-                class: helpers::map_storage_class(space),
-            },
-            crate::TypeInner::ValuePointer {
-                size: Some(size),
-                scalar,
-                space,
-            } => LocalType::LocalPointer {
-                base: NumericType::Vector { size, scalar },
-                class: helpers::map_storage_class(space),
-            },
-            crate::TypeInner::ValuePointer {
-                size: None,
-                scalar,
-                space,
-            } => LocalType::LocalPointer {
-                base: NumericType::Scalar(scalar),
-                class: helpers::map_storage_class(space),
-            },
-            crate::TypeInner::Image {
-                dim,
-                arrayed,
-                class,
-            } => LocalType::Image(LocalImageType::from_inner(dim, arrayed, class)),
-            crate::TypeInner::Sampler { comparison: _ } => LocalType::Sampler,
-            crate::TypeInner::AccelerationStructure => LocalType::AccelerationStructure,
-            crate::TypeInner::RayQuery => LocalType::RayQuery,
-            crate::TypeInner::Array { .. }
-            | crate::TypeInner::Struct { .. }
-            | crate::TypeInner::BindingArray { .. } => return None,
-        })
-    }
-}
-
 #[derive(Debug)]
 enum Dimension {
     Scalar,
@@ -475,6 +448,18 @@ enum Dimension {
     Matrix,
 }
 
+/// Key used to look up an operation which we have wrapped in a helper
+/// function, which should be called instead of directly emitting code
+/// for the expression. See [`Writer::wrapped_functions`].
+#[derive(Debug, Eq, PartialEq, Hash)]
+enum WrappedFunction {
+    BinaryOp {
+        op: crate::BinaryOperator,
+        left_type_id: Word,
+        right_type_id: Word,
+    },
+}
+
 /// A map from evaluated [`Expression`](crate::Expression)s to their SPIR-V ids.
 ///
 /// When we emit code to evaluate a given `Expression`, we record the
@@ -694,6 +679,8 @@ struct BlockContext<'w> {
 
     /// Tracks the constness of `Expression`s residing in `self.ir_function.expressions`
     expression_constness: ExpressionConstnessTracker,
+
+    force_loop_bounding: bool,
 }
 
 impl BlockContext<'_> {
@@ -705,6 +692,10 @@ impl BlockContext<'_> {
         self.writer.get_type_id(lookup_type)
     }
 
+    fn get_handle_type_id(&mut self, handle: Handle<crate::Type>) -> Word {
+        self.writer.get_handle_type_id(handle)
+    }
+
     fn get_expression_type_id(&mut self, tr: &TypeResolution) -> Word {
         self.writer.get_expression_type_id(tr)
     }
@@ -718,8 +709,12 @@ impl BlockContext<'_> {
             .get_constant_scalar(crate::Literal::I32(scope as _))
     }
 
-    fn get_pointer_id(&mut self, handle: Handle<crate::Type>, class: spirv::StorageClass) -> Word {
-        self.writer.get_pointer_id(handle, class)
+    fn get_pointer_type_id(&mut self, base: Word, class: spirv::StorageClass) -> Word {
+        self.writer.get_pointer_type_id(base, class)
+    }
+
+    fn get_numeric_type_id(&mut self, numeric: NumericType) -> Word {
+        self.writer.get_numeric_type_id(numeric)
     }
 }
 
@@ -747,11 +742,16 @@ pub struct Writer {
     flags: WriterFlags,
     bounds_check_policies: BoundsCheckPolicies,
     zero_initialize_workgroup_memory: ZeroInitializeWorkgroupMemoryMode,
+    force_loop_bounding: bool,
     void_type: Word,
     //TODO: convert most of these into vectors, addressable by handle indices
     lookup_type: crate::FastHashMap<LookupType, Word>,
     lookup_function: crate::FastHashMap<Handle<crate::Function>, Word>,
     lookup_function_type: crate::FastHashMap<LookupFunctionType, Word>,
+    /// Operations which have been wrapped in a helper function. The value is
+    /// the ID of the function, which should be called instead of emitting code
+    /// for the operation directly.
+    wrapped_functions: crate::FastHashMap<WrappedFunction, Word>,
     /// Indexed by const-expression handle indexes
     constant_ids: HandleVec<crate::Expression, Word>,
     cached_constants: crate::FastHashMap<CachedConstant, Word>,
@@ -766,6 +766,8 @@ pub struct Writer {
 
     // Just a temporary list of SPIR-V ids
     temp_list: Vec<Word>,
+
+    ray_get_intersection_function: Option<Word>,
 }
 
 bitflags::bitflags! {
@@ -809,7 +811,7 @@ pub struct BindingInfo {
 }
 
 // Using `BTreeMap` instead of `HashMap` so that we can hash itself.
-pub type BindingMap = std::collections::BTreeMap<crate::ResourceBinding, BindingInfo>;
+pub type BindingMap = alloc::collections::BTreeMap<crate::ResourceBinding, BindingInfo>;
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum ZeroInitializeWorkgroupMemoryMode {
@@ -844,10 +846,14 @@ pub struct Options<'a> {
     /// Dictates the way workgroup variables should be zero initialized
     pub zero_initialize_workgroup_memory: ZeroInitializeWorkgroupMemoryMode,
 
+    /// If set, loops will have code injected into them, forcing the compiler
+    /// to think the number of iterations is bounded.
+    pub force_loop_bounding: bool,
+
     pub debug_info: Option<DebugInfo<'a>>,
 }
 
-impl<'a> Default for Options<'a> {
+impl Default for Options<'_> {
     fn default() -> Self {
         let mut flags = WriterFlags::ADJUST_COORDINATE_SPACE
             | WriterFlags::LABEL_VARYINGS
@@ -862,6 +868,7 @@ impl<'a> Default for Options<'a> {
             capabilities: None,
             bounds_check_policies: BoundsCheckPolicies::default(),
             zero_initialize_workgroup_memory: ZeroInitializeWorkgroupMemoryMode::Polyfill,
+            force_loop_bounding: true,
             debug_info: None,
         }
     }
