@@ -9,10 +9,9 @@ mod outline;
 #[allow(unused_imports)]
 use core_maths::CoreFloat;
 
-use deltas::AvailableVarMetrics;
 pub use hint::{HintError, HintInstance, HintOutline};
 pub use outline::{Outline, ScaledOutline};
-use raw::FontRef;
+use raw::{FontRef, ReadError};
 
 use super::{DrawError, GlyphHMetrics, Hinting};
 use crate::GLYF_COMPOSITE_RECURSION_LIMIT;
@@ -54,25 +53,21 @@ pub struct Outlines<'a> {
     glyph_count: u16,
     units_per_em: u16,
     os2_vmetrics: [i16; 2],
-    var_metrics: AvailableVarMetrics,
     prefer_interpreter: bool,
+    pub(crate) fractional_size_hinting: bool,
 }
 
 impl<'a> Outlines<'a> {
     pub fn new(font: &FontRef<'a>) -> Option<Self> {
-        let loca = font.loca(None).ok()?;
+        let head = font.head().ok()?;
+        // If bit 3 of head.flags is set, then we round ppems when
+        // scaling
+        let fractional_size_hinting = !head
+            .flags()
+            .contains(read_fonts::tables::head::Flags::FORCE_INTEGER_PPEM);
+        let loca = font.loca(Some(head.index_to_loc_format() == 1)).ok()?;
         let glyf = font.glyf().ok()?;
         let glyph_metrics = GlyphHMetrics::new(font)?;
-        let var_metrics = match glyph_metrics.hvar.as_ref() {
-            Some(hvar) => {
-                if hvar.lsb_mapping().is_some() {
-                    AvailableVarMetrics::All
-                } else {
-                    AvailableVarMetrics::Advances
-                }
-            }
-            None => AvailableVarMetrics::None,
-        };
         let (
             glyph_count,
             max_function_defs,
@@ -137,8 +132,8 @@ impl<'a> Outlines<'a> {
             glyph_count,
             units_per_em: font.head().ok()?.units_per_em(),
             os2_vmetrics,
-            var_metrics,
             prefer_interpreter,
+            fractional_size_hinting,
         })
     }
 
@@ -185,6 +180,19 @@ impl<'a> Outlines<'a> {
         }
         (false, F26Dot6::from_bits(0x10000))
     }
+
+    pub fn compute_hinted_scale(&self, ppem: Option<f32>) -> (bool, F26Dot6) {
+        if let Some(ppem) = ppem {
+            if !self.fractional_size_hinting {
+                // Apply a fixed point round to ppem if the font doesn't
+                // support fractional scaling and hinting was requested.
+                // FreeType does the same.
+                // See <https://gitlab.freedesktop.org/freetype/freetype/-/blob/57617782464411201ce7bbc93b086c1b4d7d84a5/src/truetype/ttobjs.c#L1424>
+                return self.compute_scale(Some(F26Dot6::from_f64(ppem as f64).round().to_f32()));
+            }
+        }
+        self.compute_scale(ppem)
+    }
 }
 
 impl Outlines<'_> {
@@ -260,8 +268,7 @@ impl Outlines<'_> {
 }
 
 trait Scaler {
-    fn coords(&self) -> &[F2Dot14];
-    fn outlines(&self) -> &Outlines;
+    fn outlines(&self) -> &Outlines<'_>;
     fn setup_phantom_points(
         &mut self,
         bounds: [i16; 4],
@@ -293,9 +300,8 @@ trait Scaler {
             _ => [0; 4],
         };
         let outlines = self.outlines();
-        let coords: &[F2Dot14] = self.coords();
-        let lsb = outlines.glyph_metrics.lsb(glyph_id, coords);
-        let advance = outlines.glyph_metrics.advance_width(glyph_id, coords);
+        let lsb = outlines.glyph_metrics.lsb(glyph_id, &[]);
+        let advance = outlines.glyph_metrics.advance_width(glyph_id, &[]);
         let [ascent, descent] = outlines.os2_vmetrics.map(|x| x as i32);
         let tsb = ascent - bounds[3] as i32;
         let vadvance = ascent - descent;
@@ -337,6 +343,7 @@ impl<'a> HarfBuzzScaler<'a> {
         ppem: Option<f32>,
         coords: &'a [F2Dot14],
     ) -> Result<Self, DrawError> {
+        outline.ensure_point_count_limit()?;
         let (is_scaled, scale) = outlines.compute_scale(ppem);
         let memory =
             HarfBuzzOutlineMemory::new(outline, buf).ok_or(DrawError::InsufficientMemory)?;
@@ -400,6 +407,7 @@ impl<'a> FreeTypeScaler<'a> {
         ppem: Option<f32>,
         coords: &'a [F2Dot14],
     ) -> Result<Self, DrawError> {
+        outline.ensure_point_count_limit()?;
         let (is_scaled, scale) = outlines.compute_scale(ppem);
         let memory = FreeTypeOutlineMemory::new(outline, buf, Hinting::None)
             .ok_or(DrawError::InsufficientMemory)?;
@@ -429,7 +437,8 @@ impl<'a> FreeTypeScaler<'a> {
         hinter: &'a HintInstance,
         pedantic_hinting: bool,
     ) -> Result<Self, DrawError> {
-        let (is_scaled, scale) = outlines.compute_scale(ppem);
+        outline.ensure_point_count_limit()?;
+        let (is_scaled, scale) = outlines.compute_hinted_scale(ppem);
         let memory = FreeTypeOutlineMemory::new(outline, buf, Hinting::Embedded)
             .ok_or(DrawError::InsufficientMemory)?;
         Ok(Self {
@@ -503,11 +512,7 @@ impl Scaler for FreeTypeScaler<'_> {
         self.phantom[3].y = self.phantom[2].y - F26Dot6::from_bits(vadvance);
     }
 
-    fn coords(&self) -> &[F2Dot14] {
-        self.coords
-    }
-
-    fn outlines(&self) -> &Outlines {
+    fn outlines(&self) -> &Outlines<'_> {
         self.outlines
     }
 
@@ -516,11 +521,8 @@ impl Scaler for FreeTypeScaler<'_> {
         // <https://gitlab.freedesktop.org/freetype/freetype/-/blob/57617782464411201ce7bbc93b086c1b4d7d84a5/src/truetype/ttgload.c#L1572>
         let scale = self.scale;
         let mut unscaled = self.phantom.map(|point| point.map(|x| x.to_bits()));
-        if self.outlines.glyph_metrics.hvar.is_none()
-            && self.outlines.gvar.is_some()
-            && !self.coords.is_empty()
-        {
-            if let Ok(deltas) = self.outlines.gvar.as_ref().unwrap().phantom_point_deltas(
+        if self.outlines.gvar.is_some() && !self.coords.is_empty() {
+            if let Ok(Some(deltas)) = self.outlines.gvar.as_ref().unwrap().phantom_point_deltas(
                 &self.outlines.glyf,
                 &self.outlines.loca,
                 self.coords,
@@ -583,9 +585,19 @@ impl Scaler for FreeTypeScaler<'_> {
             .contours
             .get_mut(contours_start..contours_end)
             .ok_or(InsufficientMemory)?;
-        // Read the contour end points.
+        // Read the contour end points, ensuring that they are properly
+        // ordered.
+        let mut last_end_pt = 0;
         for (end_pt, contour) in contour_end_pts.iter().zip(contours.iter_mut()) {
-            *contour = end_pt.get();
+            let end_pt = end_pt.get();
+            if end_pt < last_end_pt {
+                return Err(ReadError::MalformedData(
+                    "unordered contour end points in TrueType glyph",
+                )
+                .into());
+            }
+            last_end_pt = end_pt;
+            *contour = end_pt;
         }
         // Adjust the running point/contour total counts
         self.point_count += point_count;
@@ -613,16 +625,7 @@ impl Scaler for FreeTypeScaler<'_> {
                 .iup_buffer
                 .get_mut(..point_count + PHANTOM_POINT_COUNT)
                 .ok_or(InsufficientMemory)?;
-            if deltas::simple_glyph(
-                gvar,
-                glyph_id,
-                self.coords,
-                self.outlines.var_metrics,
-                glyph,
-                iup_buffer,
-                deltas,
-            )
-            .is_ok()
+            if deltas::simple_glyph(gvar, glyph_id, self.coords, glyph, iup_buffer, deltas).is_ok()
             {
                 have_deltas = true;
             }
@@ -634,7 +637,7 @@ impl Scaler for FreeTypeScaler<'_> {
             if have_deltas {
                 for ((point, unscaled), delta) in scaled
                     .iter_mut()
-                    .zip(unscaled.iter_mut())
+                    .zip(unscaled.iter())
                     .zip(self.memory.deltas.iter())
                 {
                     let delta = delta.map(Fixed::to_f26dot6);
@@ -643,6 +646,20 @@ impl Scaler for FreeTypeScaler<'_> {
                     // extra shift.
                     *point = scaled.map(|v| F26Dot6::from_bits(v.to_i32()));
                 }
+                // FreeType applies different rounding to HVAR deltas. Since
+                // we're only using gvar, mimic that behavior for phantom point
+                // deltas when an HVAR table is present
+                if self.outlines.glyph_metrics.hvar.is_some() {
+                    for ((point, unscaled), delta) in scaled[phantom_start..]
+                        .iter_mut()
+                        .zip(&unscaled[phantom_start..])
+                        .zip(&self.memory.deltas[phantom_start..])
+                    {
+                        let delta = delta.map(Fixed::to_i32).map(F26Dot6::from_i32);
+                        let scaled = (unscaled.map(F26Dot6::from_i32) + delta) * scale;
+                        *point = scaled.map(|v| F26Dot6::from_bits(v.to_i32()));
+                    }
+                }
                 if is_hinted {
                     // For hinting, we need to adjust the unscaled points as well.
                     // Round off deltas for unscaled outlines.
@@ -668,18 +685,7 @@ impl Scaler for FreeTypeScaler<'_> {
             }
         }
         // Commit our potentially modified phantom points.
-        if self.outlines.glyph_metrics.hvar.is_some() && self.is_hinted {
-            self.phantom[0] *= self.scale;
-            self.phantom[1] *= self.scale;
-        } else {
-            for (i, point) in scaled[phantom_start..]
-                .iter()
-                .enumerate()
-                .take(PHANTOM_POINT_COUNT)
-            {
-                self.phantom[i] = *point;
-            }
-        }
+        self.phantom.copy_from_slice(&scaled[phantom_start..]);
         if let (Some(hinter), true) = (self.hinter.as_ref(), is_hinted) {
             if !ins.is_empty() {
                 // Create a copy of our scaled points in original_scaled.
@@ -762,14 +768,14 @@ impl Scaler for FreeTypeScaler<'_> {
                 .get_mut(delta_base..delta_base + count)
                 .ok_or(InsufficientMemory)?;
             if deltas::composite_glyph(gvar, glyph_id, self.coords, &mut deltas[..]).is_ok() {
-                // Apply selective deltas to phantom points.
-                self.outlines.var_metrics.phantom_deltas(
-                    &mut self.phantom,
-                    deltas,
-                    |phantom, delta| {
-                        phantom.x += F26Dot6::from_bits(delta.x.to_i32());
-                    },
-                );
+                // Apply deltas to phantom points.
+                for (phantom, delta) in self
+                    .phantom
+                    .iter_mut()
+                    .zip(&deltas[deltas.len() - PHANTOM_POINT_COUNT..])
+                {
+                    *phantom += delta.map(Fixed::to_i32).map(F26Dot6::from_bits);
+                }
                 have_deltas = true;
             }
             self.component_delta_count += count;
@@ -1027,11 +1033,7 @@ impl Scaler for HarfBuzzScaler<'_> {
         self.phantom[3].y = self.phantom[2].y - vadvance as f32;
     }
 
-    fn coords(&self) -> &[F2Dot14] {
-        self.coords
-    }
-
-    fn outlines(&self) -> &Outlines {
+    fn outlines(&self) -> &Outlines<'_> {
         self.outlines
     }
 
@@ -1044,7 +1046,7 @@ impl Scaler for HarfBuzzScaler<'_> {
             && self.outlines.gvar.is_some()
             && !self.coords.is_empty()
         {
-            if let Ok(deltas) = self.outlines.gvar.as_ref().unwrap().phantom_point_deltas(
+            if let Ok(Some(deltas)) = self.outlines.gvar.as_ref().unwrap().phantom_point_deltas(
                 &self.outlines.glyf,
                 &self.outlines.loca,
                 self.coords,
@@ -1126,16 +1128,7 @@ impl Scaler for HarfBuzzScaler<'_> {
                 .iup_buffer
                 .get_mut(..point_count + PHANTOM_POINT_COUNT)
                 .ok_or(InsufficientMemory)?;
-            if deltas::simple_glyph(
-                gvar,
-                glyph_id,
-                self.coords,
-                self.outlines.var_metrics,
-                glyph,
-                iup_buffer,
-                deltas,
-            )
-            .is_ok()
+            if deltas::simple_glyph(gvar, glyph_id, self.coords, glyph, iup_buffer, deltas).is_ok()
             {
                 for (point, delta) in points.iter_mut().zip(deltas) {
                     *point += *delta;
@@ -1182,14 +1175,14 @@ impl Scaler for HarfBuzzScaler<'_> {
                 .get_mut(delta_base..delta_base + count)
                 .ok_or(InsufficientMemory)?;
             if deltas::composite_glyph(gvar, glyph_id, self.coords, &mut deltas[..]).is_ok() {
-                // Apply selective deltas to phantom points.
-                self.outlines.var_metrics.phantom_deltas(
-                    &mut self.phantom,
-                    deltas,
-                    |phantom, delta| {
-                        phantom.x += delta.x;
-                    },
-                );
+                // Apply deltas to phantom points.
+                for (phantom, delta) in self
+                    .phantom
+                    .iter_mut()
+                    .zip(&deltas[deltas.len() - PHANTOM_POINT_COUNT..])
+                {
+                    *phantom += *delta;
+                }
                 have_deltas = true;
             }
             self.component_delta_count += count;
@@ -1303,33 +1296,17 @@ fn map_point(transform: [f32; 6], p: Point<f32>) -> Point<f32> {
     }
 }
 
-impl AvailableVarMetrics {
-    /// Calls `f` for each combination of phantom point and its associated
-    /// delta based on the available metrics present in `self`.
-    fn phantom_deltas<P, D>(
-        self,
-        phantom: &mut [Point<P>; 4],
-        deltas: &[Point<D>],
-        f: impl Fn(&mut Point<P>, &Point<D>),
-    ) {
-        match self {
-            Self::None => {
-                f(&mut phantom[0], &deltas[deltas.len() - 4]);
-                f(&mut phantom[1], &deltas[deltas.len() - 3]);
-            }
-            Self::Advances => {
-                f(&mut phantom[0], &deltas[deltas.len() - 4]);
-            }
-            Self::All => {}
-        }
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::MetadataProvider;
-    use read_fonts::{FontRef, TableProvider};
+    use raw::{
+        tables::{
+            glyf::{CompositeGlyphFlags, Glyf, SimpleGlyphFlags},
+            loca::Loca,
+        },
+        FontRead, FontRef, TableProvider,
+    };
 
     #[test]
     fn overlap_flags() {
@@ -1364,7 +1341,7 @@ mod tests {
     #[test]
     fn empty_glyph_advance() {
         let font = FontRef::new(font_test_data::HVAR_WITH_TRUNCATED_ADVANCE_INDEX_MAP).unwrap();
-        let mut outlines = Outlines::new(&font).unwrap();
+        let outlines = Outlines::new(&font).unwrap();
         let coords = [F2Dot14::from_f32(0.5)];
         let ppem = Some(24.0);
         let gid = font.charmap().map(' ').unwrap();
@@ -1375,17 +1352,8 @@ mod tests {
         let scaler =
             FreeTypeScaler::unhinted(&outlines, &outline, &mut buf, ppem, &coords).unwrap();
         let scaled = scaler.scale(&outline.glyph, gid).unwrap();
-        let advance_hvar = scaled.adjusted_advance_width();
-        // Set HVAR to None and mark var metrics as missing so we pull deltas from gvar
-        outlines.glyph_metrics.hvar = None;
-        outlines.var_metrics = AvailableVarMetrics::None;
-        let scaler =
-            FreeTypeScaler::unhinted(&outlines, &outline, &mut buf, ppem, &coords).unwrap();
-        let scaled = scaler.scale(&outline.glyph, gid).unwrap();
-        let advance_gvar = scaled.adjusted_advance_width();
-        // Make sure we have an advance and that the two are the same
-        assert!(advance_hvar != F26Dot6::ZERO);
-        assert_eq!(advance_hvar, advance_gvar);
+        let advance = scaled.adjusted_advance_width();
+        assert!(advance != F26Dot6::ZERO);
     }
 
     #[test]
@@ -1398,30 +1366,73 @@ mod tests {
         assert_eq!(outline.points, PHANTOM_POINT_COUNT);
     }
 
-    // Pull metric deltas from gvar when hvar is not present
-    // <https://github.com/googlefonts/fontations/issues/1311>
+    // fuzzer overflow for composite glyph with too many total points
+    // <https://issues.oss-fuzz.com/issues/391753684
     #[test]
-    fn missing_hvar_advance() {
-        let font = FontRef::new(font_test_data::HVAR_WITH_TRUNCATED_ADVANCE_INDEX_MAP).unwrap();
+    fn composite_with_too_many_points() {
+        let font = FontRef::new(font_test_data::GLYF_COMPONENTS).unwrap();
         let mut outlines = Outlines::new(&font).unwrap();
-        let coords = [F2Dot14::from_f32(0.5)];
-        let ppem = Some(24.0);
-        let gid = font.charmap().map('A').unwrap();
+        // Hack glyf and loca to build a glyph that contains more than 64k
+        // total points
+        let mut glyf_buf = font_test_data::bebuffer::BeBuffer::new();
+        // Make a component glyph with 40k points so we overflow the
+        // total limit in a composite
+        let simple_glyph_point_count = 40000;
+        glyf_buf = glyf_buf.push(1u16); // number of contours
+        glyf_buf = glyf_buf.extend([0i16; 4]); // bbox
+        glyf_buf = glyf_buf.push((simple_glyph_point_count - 1) as u16); // contour ends
+        glyf_buf = glyf_buf.push(0u16); // instruction count
+        for _ in 0..simple_glyph_point_count {
+            glyf_buf =
+                glyf_buf.push(SimpleGlyphFlags::X_SHORT_VECTOR | SimpleGlyphFlags::Y_SHORT_VECTOR);
+        }
+        // x/y coords
+        for _ in 0..simple_glyph_point_count * 2 {
+            glyf_buf = glyf_buf.push(0u8);
+        }
+        let glyph0_end = glyf_buf.len();
+        // Now make a composite with two components
+        glyf_buf = glyf_buf.push(-1i16); // negative signifies composite
+        glyf_buf = glyf_buf.extend([0i16; 4]); // bbox
+        for i in 0..2 {
+            let flags = if i == 0 {
+                CompositeGlyphFlags::MORE_COMPONENTS | CompositeGlyphFlags::ARGS_ARE_XY_VALUES
+            } else {
+                CompositeGlyphFlags::ARGS_ARE_XY_VALUES
+            };
+            glyf_buf = glyf_buf.push(flags); // component flag
+            glyf_buf = glyf_buf.push(0u16); // component gid
+            glyf_buf = glyf_buf.extend([0u8; 2]); // x/y offset
+        }
+        let glyph1_end = glyf_buf.len();
+        outlines.glyf = Glyf::read(glyf_buf.data().into()).unwrap();
+        // Now create a loca table
+        let mut loca_buf = font_test_data::bebuffer::BeBuffer::new();
+        loca_buf = loca_buf.extend([0u32, glyph0_end as u32, glyph1_end as u32]);
+        outlines.loca = Loca::read(loca_buf.data().into(), true).unwrap();
+        let gid = GlyphId::new(1);
         let outline = outlines.outline(gid).unwrap();
-        let mut buf = [0u8; 1024];
-        let scaler =
-            FreeTypeScaler::unhinted(&outlines, &outline, &mut buf, ppem, &coords).unwrap();
-        let scaled = scaler.scale(&outline.glyph, gid).unwrap();
-        let advance_hvar = scaled.adjusted_advance_width();
-        // Set HVAR to None and mark var metrics as missing so we pull deltas from gvar
-        outlines.glyph_metrics.hvar = None;
-        outlines.var_metrics = AvailableVarMetrics::None;
-        let scaler =
-            FreeTypeScaler::unhinted(&outlines, &outline, &mut buf, ppem, &coords).unwrap();
-        let scaled = scaler.scale(&outline.glyph, gid).unwrap();
-        let advance_gvar = scaled.adjusted_advance_width();
-        // Make sure we have an advance and that the two are the same
-        assert!(advance_hvar != F26Dot6::ZERO);
-        assert_eq!(advance_hvar, advance_gvar);
+        let mut mem_buf = vec![0u8; outline.required_buffer_size(Default::default())];
+        // This outline has more than 64k points...
+        assert!(outline.points > u16::MAX as usize);
+        let result = FreeTypeScaler::unhinted(&outlines, &outline, &mut mem_buf, None, &[]);
+        // And we get an error instead of an overflow panic
+        assert!(matches!(result, Err(DrawError::TooManyPoints(_))));
+    }
+
+    #[test]
+    fn fractional_size_hinting() {
+        let font = FontRef::from_index(font_test_data::TINOS_SUBSET, 0).unwrap();
+        let outlines = Outlines::new(&font).unwrap();
+        // Make sure we capture the correct bit
+        assert!(!outlines.fractional_size_hinting);
+        // Check proper rounding when computing scale for fractional ppem
+        // values
+        for size in [10.0, 10.2, 10.5, 10.8, 11.0] {
+            assert_eq!(
+                outlines.compute_hinted_scale(Some(size)),
+                outlines.compute_hinted_scale(Some(size.round()))
+            );
+        }
     }
 }
