@@ -9,8 +9,8 @@ pub(super) fn with_temporary_memory<R>(size: usize, mut f: impl FnMut(&mut [u8])
     fn stack_mem<const STACK_SIZE: usize, R>(size: usize, mut f: impl FnMut(&mut [u8]) -> R) -> R {
         f(&mut [0u8; STACK_SIZE][..size])
     }
-    // Use bucketed stack allocations to prevent excessive zeroing of
-    // memory
+    // Use bucketed stack allocations (up to 16k) to prevent excessive zeroing
+    // of memory
     if size <= 512 {
         stack_mem::<512, _>(size, f)
     } else if size <= 1024 {
@@ -19,6 +19,10 @@ pub(super) fn with_temporary_memory<R>(size: usize, mut f: impl FnMut(&mut [u8])
         stack_mem::<2048, _>(size, f)
     } else if size <= 4096 {
         stack_mem::<4096, _>(size, f)
+    } else if size <= 8192 {
+        stack_mem::<8192, _>(size, f)
+    } else if size <= 16384 {
+        stack_mem::<16384, _>(size, f)
     } else {
         f(&mut vec![0u8; size])
     }
