@@ -13,7 +13,8 @@
  * limitations under the License.
  */
 
-use crate::limits::MAX_WASM_CATCHES;
+use crate::limits::{MAX_WASM_CATCHES, MAX_WASM_HANDLERS};
+use crate::prelude::*;
 use crate::{BinaryReader, BinaryReaderError, FromReader, Result, ValType};
 
 /// Represents a block type.
@@ -29,8 +30,45 @@ pub enum BlockType {
     FuncType(u32),
 }
 
+/// The kind of a control flow `Frame`.
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub enum FrameKind {
+    /// A Wasm `block` control block.
+    Block,
+    /// A Wasm `if` control block.
+    If,
+    /// A Wasm `else` control block.
+    Else,
+    /// A Wasm `loop` control block.
+    Loop,
+    /// A Wasm `try` control block.
+    ///
+    /// # Note
+    ///
+    /// This belongs to the Wasm exception handling proposal.
+    TryTable,
+    /// A Wasm legacy `try` control block.
+    ///
+    /// # Note
+    ///
+    /// See: `WasmFeatures::legacy_exceptions` Note in `crates/wasmparser/src/features.rs`
+    LegacyTry,
+    /// A Wasm legacy `catch` control block.
+    ///
+    /// # Note
+    ///
+    /// See: `WasmFeatures::legacy_exceptions` Note in `crates/wasmparser/src/features.rs`
+    LegacyCatch,
+    /// A Wasm legacy `catch_all` control block.
+    ///
+    /// # Note
+    ///
+    /// See: `WasmFeatures::legacy_exceptions` Note in `crates/wasmparser/src/features.rs`
+    LegacyCatchAll,
+}
+
 /// Represents a memory immediate in a WebAssembly memory instruction.
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub struct MemArg {
     /// Alignment, stored as `n` where the actual alignment is `2^n`
     pub align: u8,
@@ -63,6 +101,16 @@ pub struct BrTable<'a> {
     pub(crate) default: u32,
 }
 
+impl PartialEq<Self> for BrTable<'_> {
+    fn eq(&self, other: &Self) -> bool {
+        self.cnt == other.cnt
+            && self.default == other.default
+            && self.reader.remaining_buffer() == other.reader.remaining_buffer()
+    }
+}
+
+impl Eq for BrTable<'_> {}
+
 /// An IEEE binary32 immediate floating point value, represented as a u32
 /// containing the bit pattern.
 ///
@@ -77,6 +125,20 @@ impl Ieee32 {
     }
 }
 
+impl From<f32> for Ieee32 {
+    fn from(value: f32) -> Self {
+        Ieee32 {
+            0: u32::from_le_bytes(value.to_le_bytes()),
+        }
+    }
+}
+
+impl From<Ieee32> for f32 {
+    fn from(bits: Ieee32) -> f32 {
+        f32::from_bits(bits.bits())
+    }
+}
+
 /// An IEEE binary64 immediate floating point value, represented as a u64
 /// containing the bit pattern.
 ///
@@ -91,6 +153,20 @@ impl Ieee64 {
     }
 }
 
+impl From<f64> for Ieee64 {
+    fn from(value: f64) -> Self {
+        Ieee64 {
+            0: u64::from_le_bytes(value.to_le_bytes()),
+        }
+    }
+}
+
+impl From<Ieee64> for f64 {
+    fn from(bits: Ieee64) -> f64 {
+        f64::from_bits(bits.bits())
+    }
+}
+
 /// Represents a 128-bit vector value.
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
 pub struct V128(pub(crate) [u8; 16]);
@@ -107,13 +183,44 @@ impl V128 {
     }
 }
 
+impl From<V128> for i128 {
+    fn from(bits: V128) -> i128 {
+        bits.i128()
+    }
+}
+
+impl From<V128> for u128 {
+    fn from(bits: V128) -> u128 {
+        u128::from_le_bytes(bits.0)
+    }
+}
+
+/// Represents the memory ordering for atomic instructions.
+///
+/// For an in-depth explanation of memory orderings, see the C++ documentation
+/// for [`memory_order`] or the Rust documentation for [`atomic::Ordering`].
+///
+/// [`memory_order`]: https://en.cppreference.com/w/cpp/atomic/memory_order
+/// [`atomic::Ordering`]: https://doc.rust-lang.org/std/sync/atomic/enum.Ordering.html
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
+pub enum Ordering {
+    /// For a load, it acquires; this orders all operations before the last
+    /// "releasing" store. For a store, it releases; this orders all operations
+    /// before it at the next "acquiring" load.
+    AcqRel,
+    /// Like `AcqRel` but all threads see all sequentially consistent operations
+    /// in the same order.
+    SeqCst,
+}
+
 macro_rules! define_operator {
-    ($(@$proposal:ident $op:ident $({ $($payload:tt)* })? => $visit:ident)*) => {
+    ($(@$proposal:ident $op:ident $({ $($payload:tt)* })? => $visit:ident ($($ann:tt)*))*) => {
         /// Instructions as defined [here].
         ///
         /// [here]: https://webassembly.github.io/spec/core/binary/instructions.html
-        #[derive(Debug, Clone)]
+        #[derive(Debug, Clone, Eq, PartialEq)]
         #[allow(missing_docs)]
+        #[non_exhaustive]
         pub enum Operator<'a> {
             $(
                 $op $({ $($payload)* })?,
@@ -121,12 +228,12 @@ macro_rules! define_operator {
         }
     }
 }
-for_each_operator!(define_operator);
+crate::for_each_operator!(define_operator);
 
 /// A reader for a core WebAssembly function's operators.
 #[derive(Clone)]
 pub struct OperatorsReader<'a> {
-    pub(crate) reader: BinaryReader<'a>,
+    reader: BinaryReader<'a>,
 }
 
 impl<'a> OperatorsReader<'a> {
@@ -144,15 +251,6 @@ impl<'a> OperatorsReader<'a> {
         self.reader.original_position()
     }
 
-    /// Whether or not to allow 64-bit memory arguments in the
-    /// the operators being read.
-    ///
-    /// This is intended to be `true` when support for the memory64
-    /// WebAssembly proposal is also enabled.
-    pub fn allow_memarg64(&mut self, allow: bool) {
-        self.reader.allow_memarg64(allow);
-    }
-
     /// Ensures the reader is at the end.
     ///
     /// This function returns an error if there is extra data after the operators.
@@ -199,6 +297,12 @@ impl<'a> OperatorsReader<'a> {
     pub fn get_binary_reader(&self) -> BinaryReader<'a> {
         self.reader.clone()
     }
+
+    /// Returns whether there is an `end` opcode followed by eof remaining in
+    /// this reader.
+    pub fn is_end_then_eof(&self) -> bool {
+        self.reader.is_end_then_eof()
+    }
 }
 
 impl<'a> IntoIterator for OperatorsReader<'a> {
@@ -209,10 +313,11 @@ impl<'a> IntoIterator for OperatorsReader<'a> {
     ///
     /// # Examples
     /// ```
-    /// use wasmparser::{Operator, CodeSectionReader, Result};
+    /// # use wasmparser::{Operator, CodeSectionReader, Result, BinaryReader};
     /// # let data: &[u8] = &[
     /// #     0x01, 0x03, 0x00, 0x01, 0x0b];
-    /// let code_reader = CodeSectionReader::new(data, 0).unwrap();
+    /// let reader = BinaryReader::new(data, 0);
+    /// let code_reader = CodeSectionReader::new(reader).unwrap();
     /// for body in code_reader {
     ///     let body = body.expect("function body");
     ///     let mut op_reader = body.get_operators_reader().expect("op reader");
@@ -264,10 +369,11 @@ impl<'a> Iterator for OperatorsIteratorWithOffsets<'a> {
     ///
     /// # Examples
     /// ```
-    /// use wasmparser::{Operator, CodeSectionReader, Result};
+    /// use wasmparser::{Operator, CodeSectionReader, Result, BinaryReader};
     /// # let data: &[u8] = &[
     /// #     0x01, 0x03, 0x00, /* offset = 23 */ 0x01, 0x0b];
-    /// let code_reader = CodeSectionReader::new(data, 20).unwrap();
+    /// let reader = BinaryReader::new(data, 20);
+    /// let code_reader = CodeSectionReader::new(reader).unwrap();
     /// for body in code_reader {
     ///     let body = body.expect("function body");
     ///     let mut op_reader = body.get_operators_reader().expect("op reader");
@@ -290,7 +396,7 @@ impl<'a> Iterator for OperatorsIteratorWithOffsets<'a> {
 }
 
 macro_rules! define_visit_operator {
-    ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident)*) => {
+    ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident ($($ann:tt)*))*) => {
         $(
             fn $visit(&mut self $($(,$arg: $argty)*)?) -> Self::Output;
         )*
@@ -313,23 +419,89 @@ pub trait VisitOperator<'a> {
     /// implement [`VisitOperator`] on their own.
     fn visit_operator(&mut self, op: &Operator<'a>) -> Self::Output {
         macro_rules! visit_operator {
-            ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident)*) => {
+            ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident ($($ann:tt)*))*) => {{
                 match op {
-                    $(
-                        Operator::$op $({ $($arg),* })? => self.$visit($($($arg.clone()),*)?),
-                    )*
+                    $( Operator::$op $({ $($arg),* })? => self.$visit($($($arg.clone()),*)?), )*
+                    #[cfg(feature = "simd")]
+                    other => visit_simd_operator(self, other),
                 }
-            }
-
+            }};
         }
-        for_each_operator!(visit_operator)
+        crate::for_each_visit_operator!(visit_operator)
     }
 
-    for_each_operator!(define_visit_operator);
+    /// Returns a mutable reference to a [`VisitSimdOperator`] visitor.
+    ///
+    /// - If an implementer does _not_ want to support Wasm `simd` proposal
+    ///   nothing has to be done since the default implementation already suffices.
+    /// - If an implementer _does_ want to support Wasm `simd` proposal this
+    ///   method usually is implemented as `Some(self)` where the implementing
+    ///   type (`Self`) typically also implements `VisitSimdOperator`.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # macro_rules! define_visit_operator {
+    /// #     ($( @$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident ($($ann:tt)*))*) => {
+    /// #         $( fn $visit(&mut self $($(,$arg: $argty)*)?) {} )*
+    /// #     }
+    /// # }
+    /// # use wasmparser::{VisitOperator, VisitSimdOperator};
+    /// pub struct MyVisitor;
+    ///
+    /// impl<'a> VisitOperator<'a> for MyVisitor {
+    ///     type Output = ();
+    ///
+    ///     fn simd_visitor(&mut self) -> Option<&mut dyn VisitSimdOperator<'a, Output = Self::Output>> {
+    ///         Some(self)
+    ///     }
+    ///
+    ///     // implement remaining visitation methods here ...
+    ///     # wasmparser::for_each_visit_operator!(define_visit_operator);
+    /// }
+    ///
+    /// impl VisitSimdOperator<'_> for MyVisitor {
+    ///     // implement SIMD visitation methods here ...
+    ///     # wasmparser::for_each_visit_simd_operator!(define_visit_operator);
+    /// }
+    /// ```
+    #[cfg(feature = "simd")]
+    fn simd_visitor(&mut self) -> Option<&mut dyn VisitSimdOperator<'a, Output = Self::Output>> {
+        None
+    }
+
+    crate::for_each_visit_operator!(define_visit_operator);
+}
+
+/// Special handler for visiting `simd` and `relaxed-simd` [`Operator`] variants.
+#[cfg(feature = "simd")]
+fn visit_simd_operator<'a, V>(visitor: &mut V, op: &Operator<'a>) -> V::Output
+where
+    V: VisitOperator<'a> + ?Sized,
+{
+    let Some(simd_visitor) = visitor.simd_visitor() else {
+        panic!("missing SIMD visitor to visit operator: {op:?}")
+    };
+    macro_rules! visit_simd_operator {
+        ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident ($($ann:tt)*))*) => {{
+            match op {
+                $( Operator::$op $({ $($arg),* })? => simd_visitor.$visit($($($arg.clone()),*)?), )*
+                unexpected => unreachable!("unexpected non-SIMD operator: {unexpected:?}"),
+            }
+        }};
+    }
+    crate::for_each_visit_simd_operator!(visit_simd_operator)
+}
+
+/// Trait implemented by types that can visit all Wasm `simd` and `relaxed-simd` [`Operator`]s.
+#[cfg(feature = "simd")]
+#[allow(missing_docs)]
+pub trait VisitSimdOperator<'a>: VisitOperator<'a> {
+    crate::for_each_visit_simd_operator!(define_visit_operator);
 }
 
 macro_rules! define_visit_operator_delegate {
-    ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident)*) => {
+    ($(@$proposal:ident $op:ident $({ $($arg:ident: $argty:ty),* })? => $visit:ident ($($ann:tt)*))*) => {
         $(
             fn $visit(&mut self $($(,$arg: $argty)*)?) -> Self::Output {
                 V::$visit(&mut *self, $($($arg),*)?)
@@ -343,7 +515,16 @@ impl<'a, 'b, V: VisitOperator<'a> + ?Sized> VisitOperator<'a> for &'b mut V {
     fn visit_operator(&mut self, op: &Operator<'a>) -> Self::Output {
         V::visit_operator(*self, op)
     }
-    for_each_operator!(define_visit_operator_delegate);
+    #[cfg(feature = "simd")]
+    fn simd_visitor(&mut self) -> Option<&mut dyn VisitSimdOperator<'a, Output = V::Output>> {
+        V::simd_visitor(*self)
+    }
+    crate::for_each_visit_operator!(define_visit_operator_delegate);
+}
+
+#[cfg(feature = "simd")]
+impl<'a, 'b, V: VisitSimdOperator<'a> + ?Sized> VisitSimdOperator<'a> for &'b mut V {
+    crate::for_each_visit_simd_operator!(define_visit_operator_delegate);
 }
 
 impl<'a, V: VisitOperator<'a> + ?Sized> VisitOperator<'a> for Box<V> {
@@ -351,11 +532,20 @@ impl<'a, V: VisitOperator<'a> + ?Sized> VisitOperator<'a> for Box<V> {
     fn visit_operator(&mut self, op: &Operator<'a>) -> Self::Output {
         V::visit_operator(&mut *self, op)
     }
-    for_each_operator!(define_visit_operator_delegate);
+    #[cfg(feature = "simd")]
+    fn simd_visitor(&mut self) -> Option<&mut dyn VisitSimdOperator<'a, Output = V::Output>> {
+        V::simd_visitor(&mut *self)
+    }
+    crate::for_each_visit_operator!(define_visit_operator_delegate);
+}
+
+#[cfg(feature = "simd")]
+impl<'a, V: VisitSimdOperator<'a> + ?Sized> VisitSimdOperator<'a> for Box<V> {
+    crate::for_each_visit_simd_operator!(define_visit_operator_delegate);
 }
 
 /// A `try_table` entries representation.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub struct TryTable {
     /// The block type describing the try block itself.
     pub ty: BlockType,
@@ -364,7 +554,7 @@ pub struct TryTable {
 }
 
 /// Catch clauses that can be specified in [`TryTable`].
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
 #[allow(missing_docs)]
 pub enum Catch {
     /// Equivalent of `catch`
@@ -409,3 +599,54 @@ impl<'a> FromReader<'a> for Catch {
         })
     }
 }
+
+/// A representation of dispatch tables on `resume` and `resume_throw`
+/// instructions.
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct ResumeTable {
+    /// Either the outer blocks which will handle suspensions or
+    /// "switch-to" handlers.
+    pub handlers: Vec<Handle>,
+}
+
+/// Handle clauses that can be specified in [`ResumeTable`].
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+#[allow(missing_docs)]
+pub enum Handle {
+    /// Equivalent of `(on $tag $lbl)`.
+    OnLabel { tag: u32, label: u32 },
+    /// Equivalent of `(on $tag switch)`.
+    OnSwitch { tag: u32 },
+}
+
+impl ResumeTable {
+    /// Returns the number of entries in the table.
+    pub fn len(&self) -> usize {
+        self.handlers.len()
+    }
+}
+
+impl<'a> FromReader<'a> for ResumeTable {
+    fn from_reader(reader: &mut BinaryReader<'a>) -> Result<Self> {
+        let handlers = reader
+            .read_iter(MAX_WASM_HANDLERS, "resume table")?
+            .collect::<Result<_>>()?;
+        let table = ResumeTable { handlers };
+        Ok(table)
+    }
+}
+
+impl<'a> FromReader<'a> for Handle {
+    fn from_reader(reader: &mut BinaryReader<'a>) -> Result<Self> {
+        Ok(match reader.read_u8()? {
+            0x00 => Handle::OnLabel {
+                tag: reader.read_var_u32()?,
+                label: reader.read_var_u32()?,
+            },
+            0x01 => Handle::OnSwitch {
+                tag: reader.read_var_u32()?,
+            },
+            x => return reader.invalid_leading_byte(x, "on clause"),
+        })
+    }
+}
