@@ -50,12 +50,25 @@ pub fn print_bytes(wasm: impl AsRef<[u8]>) -> Result<String> {
 ///
 /// This structure is used to control the overal structure of how wasm binaries
 /// are printed and tweaks various ways that configures the output.
-#[derive(Debug, Default)]
+#[derive(Debug)]
 pub struct Config {
     print_offsets: bool,
     print_skeleton: bool,
     name_unnamed: bool,
     fold_instructions: bool,
+    indent_text: String,
+}
+
+impl Default for Config {
+    fn default() -> Self {
+        Self {
+            print_offsets: false,
+            print_skeleton: false,
+            name_unnamed: false,
+            fold_instructions: false,
+            indent_text: "  ".to_string(),
+        }
+    }
 }
 
 /// This structure is the actual structure that prints WebAssembly binaries.
@@ -226,10 +239,21 @@ impl Config {
         self
     }
 
-    /// Prints a WebAssembly binary into a `String`
+    /// Select the string to use when indenting.
     ///
-    /// This function takes an entire `wasm` binary blob and will print it to
-    /// the WebAssembly Text Format and return the result as a `String`.
+    /// The indent allowed here are arbitrary and unchecked. You should enter
+    /// blank text like `" "` or `"\t"`, rather than something like `"(;;)"`.
+    ///
+    /// The default setting is double spaces `" "`
+    pub fn indent_text(&mut self, text: impl Into<String>) -> &mut Self {
+        self.indent_text = text.into();
+        self
+    }
+
+    /// Print a WebAssembly binary.
+    ///
+    /// This function takes an entire `wasm` binary blob and prints it to the
+    /// `result` in the WebAssembly Text Format.
     pub fn print(&self, wasm: &[u8], result: &mut impl Print) -> Result<()> {
         Printer {
             config: self,
@@ -1267,9 +1291,7 @@ impl Printer<'_, '_> {
         state: &mut State,
         body: &FunctionBody<'_>,
     ) -> Result<()> {
-        let mut body = body.get_binary_reader();
-        let offset = body.original_position();
-        self.newline(offset)?;
+        self.newline(body.get_binary_reader().original_position())?;
         self.start_group("func ")?;
         let func_idx = state.core.funcs;
         self.print_name(&state.core.func_names, func_idx)?;
@@ -1295,7 +1317,7 @@ impl Printer<'_, '_> {
         if self.config.print_skeleton {
             self.result.write_str(" ...")?;
         } else {
-            self.print_func_body(state, func_idx, params, &mut body, &hints)?;
+            self.print_func_body(state, func_idx, params, &body, &hints)?;
         }
 
         self.end_group()?;
@@ -1308,17 +1330,18 @@ impl Printer<'_, '_> {
         state: &mut State,
         func_idx: u32,
         params: u32,
-        body: &mut BinaryReader<'_>,
+        body: &FunctionBody<'_>,
         branch_hints: &[(usize, BranchHint)],
     ) -> Result<()> {
         let mut first = true;
         let mut local_idx = 0;
         let mut locals = NamedLocalPrinter::new("local");
-        let func_start = body.original_position();
-        for _ in 0..body.read_var_u32()? {
-            let offset = body.original_position();
-            let cnt = body.read_var_u32()?;
-            let ty = body.read()?;
+        let mut reader = body.get_binary_reader();
+        let func_start = reader.original_position();
+        for _ in 0..reader.read_var_u32()? {
+            let offset = reader.original_position();
+            let cnt = reader.read_var_u32()?;
+            let ty = reader.read()?;
             if MAX_LOCALS
                 .checked_sub(local_idx)
                 .and_then(|s| s.checked_sub(cnt))
@@ -1347,11 +1370,11 @@ impl Printer<'_, '_> {
             let mut folded_printer = PrintOperatorFolded::new(self, state, &mut operator_state);
             folded_printer.set_offset(func_start);
             folded_printer.begin_function(func_idx)?;
-            Self::print_operators(body, branch_hints, func_start, &mut folded_printer)?;
+            Self::print_operators(&mut reader, branch_hints, func_start, &mut folded_printer)?;
             folded_printer.finalize()?;
         } else {
             let mut flat_printer = PrintOperator::new(self, state, &mut operator_state);
-            Self::print_operators(body, branch_hints, func_start, &mut flat_printer)?;
+            Self::print_operators(&mut reader, branch_hints, func_start, &mut flat_printer)?;
         }
 
         // If this was an invalid function body then the nesting may not
@@ -1361,7 +1384,7 @@ impl Printer<'_, '_> {
         // with the closing paren printed for the func.
         if self.nesting != nesting_start {
             self.nesting = nesting_start;
-            self.newline(body.original_position())?;
+            self.newline(reader.original_position())?;
         }
 
         Ok(())
@@ -1373,20 +1396,28 @@ impl Printer<'_, '_> {
         func_start: usize,
         op_printer: &mut O,
     ) -> Result<()> {
-        while !body.is_end_then_eof() {
+        let mut ops = OperatorsReader::new(body.clone());
+        while !ops.eof() {
+            if ops.is_end_then_eof() {
+                ops.read()?; // final "end" opcode terminates instruction sequence
+                ops.finish()?;
+                return Ok(());
+            }
+
             // Branch hints are stored in increasing order of their body offset
             // so print them whenever their instruction comes up.
             if let Some(((hint_offset, hint), rest)) = branch_hints.split_first() {
-                if hint.func_offset == (body.original_position() - func_start) as u32 {
+                if hint.func_offset == (ops.original_position() - func_start) as u32 {
                     branch_hints = rest;
                     op_printer.branch_hint(*hint_offset, hint.taken)?;
                 }
             }
 
-            op_printer.set_offset(body.original_position());
-            op_printer.visit_operator(body)?;
+            op_printer.set_offset(ops.original_position());
+            op_printer.visit_operator(&mut ops)?;
         }
-        Ok(())
+        ops.finish()?; // for the error message
+        bail!("unexpected end of operators");
     }
 
     fn newline(&mut self, offset: usize) -> Result<()> {
@@ -1413,7 +1444,7 @@ impl Printer<'_, '_> {
         // reasonable to avoid generating hundreds of megabytes of whitespace
         // for small-ish modules that have deep-ish nesting.
         for _ in 0..self.nesting.min(MAX_NESTING_TO_PRINT) {
-            self.result.write_str("  ")?;
+            self.result.write_str(&self.config.indent_text)?;
         }
         Ok(())
     }
