@@ -1,13 +1,13 @@
 //! Memory management for executable code.
 
 use crate::prelude::*;
-use crate::runtime::vm::{libcalls, MmapVec, UnwindRegistration};
+use crate::runtime::vm::MmapVec;
 use crate::Engine;
 use alloc::sync::Arc;
 use core::ops::Range;
 use object::endian::Endianness;
 use object::read::{elf::ElfFile64, Object, ObjectSection};
-use object::{ObjectSymbol, SectionFlags};
+use object::SectionFlags;
 use wasmtime_environ::{lookup_trap_code, obj, Trap};
 
 /// Management of executable memory within a `MmapVec`
@@ -16,7 +16,8 @@ use wasmtime_environ::{lookup_trap_code, obj, Trap};
 /// executable permissions of the contained JIT code as necessary.
 pub struct CodeMemory {
     mmap: MmapVec,
-    unwind_registration: Option<UnwindRegistration>,
+    #[cfg(has_host_compiler_backend)]
+    unwind_registration: Option<crate::runtime::vm::UnwindRegistration>,
     #[cfg(feature = "debug-builtins")]
     debug_registration: Option<crate::runtime::vm::GdbJitImageRegistration>,
     published: bool,
@@ -26,14 +27,13 @@ pub struct CodeMemory {
     has_native_debug_info: bool,
     custom_code_memory: Option<Arc<dyn CustomCodeMemory>>,
 
-    relocations: Vec<(usize, obj::LibCall)>,
-
     // Ranges within `self.mmap` of where the particular sections lie.
     text: Range<usize>,
     unwind: Range<usize>,
     trap_data: Range<usize>,
     wasm_data: Range<usize>,
     address_map_data: Range<usize>,
+    stack_map_data: Range<usize>,
     func_name_data: Range<usize>,
     info_data: Range<usize>,
     wasm_dwarf: Range<usize>,
@@ -44,12 +44,15 @@ impl Drop for CodeMemory {
         // If there is a custom code memory handler, restore the
         // original (non-executable) state of the memory.
         if let Some(mem) = self.custom_code_memory.as_ref() {
-            let text = self.text();
-            mem.unpublish_executable(text.as_ptr(), text.len())
-                .expect("Executable memory unpublish failed");
+            if self.published && self.needs_executable {
+                let text = self.text();
+                mem.unpublish_executable(text.as_ptr(), text.len())
+                    .expect("Executable memory unpublish failed");
+            }
         }
 
         // Drop the registrations before `self.mmap` since they (implicitly) refer to it.
+        #[cfg(has_host_compiler_backend)]
         let _ = self.unwind_registration.take();
         #[cfg(feature = "debug-builtins")]
         let _ = self.debug_registration.take();
@@ -109,7 +112,6 @@ impl CodeMemory {
             .map_err(obj::ObjectCrateErrorWrapper)
             .with_context(|| "failed to parse internal compilation artifact")?;
 
-        let mut relocations = Vec::new();
         let mut text = 0..0;
         let mut unwind = 0..0;
         let mut enable_branch_protection = None;
@@ -119,6 +121,7 @@ impl CodeMemory {
         let mut trap_data = 0..0;
         let mut wasm_data = 0..0;
         let mut address_map_data = 0..0;
+        let mut stack_map_data = 0..0;
         let mut func_name_data = 0..0;
         let mut info_data = 0..0;
         let mut wasm_dwarf = 0..0;
@@ -152,32 +155,18 @@ impl CodeMemory {
                         }
                     }
 
-                    // The text section might have relocations for things like
-                    // libcalls which need to be applied, so handle those here.
-                    //
-                    // Note that only a small subset of possible relocations are
-                    // handled. Only those required by the compiler side of
-                    // things are processed.
-                    for (offset, reloc) in section.relocations() {
-                        assert_eq!(reloc.kind(), object::RelocationKind::Absolute);
-                        assert_eq!(reloc.encoding(), object::RelocationEncoding::Generic);
-                        assert_eq!(usize::from(reloc.size()), core::mem::size_of::<usize>() * 8);
-                        assert_eq!(reloc.addend(), 0);
-                        let sym = match reloc.target() {
-                            object::RelocationTarget::Symbol(id) => id,
-                            other => panic!("unknown relocation target {other:?}"),
-                        };
-                        let sym = obj.symbol_by_index(sym).unwrap().name().unwrap();
-                        let libcall = obj::LibCall::from_str(sym)
-                            .unwrap_or_else(|| panic!("unknown symbol relocation: {sym}"));
-
-                        let offset = usize::try_from(offset).unwrap();
-                        relocations.push((offset, libcall));
-                    }
+                    // Assert that Cranelift hasn't inserted any calls that need to be
+                    // relocated. We avoid using things like Cranelift's floor/ceil/etc.
+                    // operators in the Wasm-to-Cranelift translator specifically to
+                    // avoid having to do any relocations here. This also ensures that
+                    // all builtins use the same trampoline mechanism.
+                    assert!(section.relocations().next().is_none());
                 }
-                UnwindRegistration::SECTION_NAME => unwind = range,
+                #[cfg(has_host_compiler_backend)]
+                crate::runtime::vm::UnwindRegistration::SECTION_NAME => unwind = range,
                 obj::ELF_WASM_DATA => wasm_data = range,
                 obj::ELF_WASMTIME_ADDRMAP => address_map_data = range,
+                obj::ELF_WASMTIME_STACK_MAP => stack_map_data = range,
                 obj::ELF_WASMTIME_TRAPS => trap_data = range,
                 obj::ELF_NAME_DATA => func_name_data = range,
                 obj::ELF_WASMTIME_INFO => info_data = range,
@@ -189,8 +178,13 @@ impl CodeMemory {
             }
         }
 
+        // require mutability even when this is turned off
+        #[cfg(not(has_host_compiler_backend))]
+        let _ = &mut unwind;
+
         Ok(Self {
             mmap,
+            #[cfg(has_host_compiler_backend)]
             unwind_registration: None,
             #[cfg(feature = "debug-builtins")]
             debug_registration: None,
@@ -205,11 +199,11 @@ impl CodeMemory {
             unwind,
             trap_data,
             address_map_data,
+            stack_map_data,
             func_name_data,
             wasm_dwarf,
             info_data,
             wasm_data,
-            relocations,
         })
     }
 
@@ -255,6 +249,12 @@ impl CodeMemory {
         &self.mmap[self.address_map_data.clone()]
     }
 
+    /// Returns the encoded stack map section used to pass to
+    /// `wasmtime_environ::StackMap::lookup`.
+    pub fn stack_map_data(&self) -> &[u8] {
+        &self.mmap[self.stack_map_data.clone()]
+    }
+
     /// Returns the contents of the `ELF_WASMTIME_INFO` section, or an empty
     /// slice if it wasn't found.
     #[inline]
@@ -297,13 +297,6 @@ impl CodeMemory {
         //   both the actual unwinding tables as well as the validity of the
         //   pointers we pass in itself.
         unsafe {
-            // First, if necessary, apply relocations. This can happen for
-            // things like libcalls which happen late in the lowering process
-            // that don't go through the Wasm-based libcalls layer that's
-            // indirected through the `VMContext`. Note that most modules won't
-            // have relocations, so this typically doesn't do anything.
-            self.apply_relocations()?;
-
             // Next freeze the contents of this image by making all of the
             // memory readonly. Nothing after this point should ever be modified
             // so commit everything. For a compiled-in-memory image this will
@@ -315,13 +308,19 @@ impl CodeMemory {
             // Note that if virtual memory is disabled this is skipped because
             // we aren't able to make it readonly, but this is just a
             // defense-in-depth measure and isn't required for correctness.
-            #[cfg(feature = "signals-based-traps")]
-            self.mmap.make_readonly(0..self.mmap.len())?;
+            #[cfg(has_virtual_memory)]
+            if self.mmap.supports_virtual_memory() {
+                self.mmap.make_readonly(0..self.mmap.len())?;
+            }
 
             // Switch the executable portion from readonly to read/execute.
             if self.needs_executable {
                 if !self.custom_publish()? {
-                    #[cfg(feature = "signals-based-traps")]
+                    if !self.mmap.supports_virtual_memory() {
+                        bail!("this target requires virtual memory to be enabled");
+                    }
+
+                    #[cfg(has_virtual_memory)]
                     {
                         let text = self.text();
 
@@ -341,8 +340,6 @@ impl CodeMemory {
                         // Flush any in-flight instructions from the pipeline
                         icache_coherence::pipeline_flush_mt().expect("Failed pipeline flush");
                     }
-                    #[cfg(not(feature = "signals-based-traps"))]
-                    bail!("this target requires virtual memory to be enabled");
                 }
             }
 
@@ -382,50 +379,27 @@ impl CodeMemory {
         }
     }
 
-    unsafe fn apply_relocations(&mut self) -> Result<()> {
-        if self.relocations.is_empty() {
-            return Ok(());
-        }
-
-        for (offset, libcall) in self.relocations.iter() {
-            let offset = self.text.start + offset;
-            let libcall = match libcall {
-                obj::LibCall::FloorF32 => libcalls::relocs::floorf32 as usize,
-                obj::LibCall::FloorF64 => libcalls::relocs::floorf64 as usize,
-                obj::LibCall::NearestF32 => libcalls::relocs::nearestf32 as usize,
-                obj::LibCall::NearestF64 => libcalls::relocs::nearestf64 as usize,
-                obj::LibCall::CeilF32 => libcalls::relocs::ceilf32 as usize,
-                obj::LibCall::CeilF64 => libcalls::relocs::ceilf64 as usize,
-                obj::LibCall::TruncF32 => libcalls::relocs::truncf32 as usize,
-                obj::LibCall::TruncF64 => libcalls::relocs::truncf64 as usize,
-                obj::LibCall::FmaF32 => libcalls::relocs::fmaf32 as usize,
-                obj::LibCall::FmaF64 => libcalls::relocs::fmaf64 as usize,
-                #[cfg(target_arch = "x86_64")]
-                obj::LibCall::X86Pshufb => libcalls::relocs::x86_pshufb as usize,
-                #[cfg(not(target_arch = "x86_64"))]
-                obj::LibCall::X86Pshufb => unreachable!(),
-            };
-            self.mmap
-                .as_mut_slice()
-                .as_mut_ptr()
-                .add(offset)
-                .cast::<usize>()
-                .write_unaligned(libcall);
-        }
-        Ok(())
-    }
-
     unsafe fn register_unwind_info(&mut self) -> Result<()> {
         if self.unwind.len() == 0 {
             return Ok(());
         }
-        let text = self.text();
-        let unwind_info = &self.mmap[self.unwind.clone()];
-        let registration =
-            UnwindRegistration::new(text.as_ptr(), unwind_info.as_ptr(), unwind_info.len())
-                .context("failed to create unwind info registration")?;
-        self.unwind_registration = Some(registration);
-        Ok(())
+        #[cfg(has_host_compiler_backend)]
+        {
+            let text = self.text();
+            let unwind_info = &self.mmap[self.unwind.clone()];
+            let registration = crate::runtime::vm::UnwindRegistration::new(
+                text.as_ptr(),
+                unwind_info.as_ptr(),
+                unwind_info.len(),
+            )
+            .context("failed to create unwind info registration")?;
+            self.unwind_registration = Some(registration);
+            return Ok(());
+        }
+        #[cfg(not(has_host_compiler_backend))]
+        {
+            bail!("should not have unwind info for non-native backend")
+        }
     }
 
     #[cfg(feature = "debug-builtins")]
