@@ -2,7 +2,9 @@ use crate::component::func::{bad_type_info, desc, LiftContext, LowerContext};
 use crate::component::matching::InstanceType;
 use crate::component::{ComponentType, Lift, Lower};
 use crate::prelude::*;
-use crate::runtime::vm::component::{ComponentInstance, InstanceFlags, ResourceTables};
+use crate::runtime::vm::component::{
+    ComponentInstance, InstanceFlags, ResourceTables, TypedResource, TypedResourceIndex,
+};
 use crate::runtime::vm::{SendSyncPtr, VMFuncRef, ValRaw};
 use crate::store::{StoreId, StoreOpaque};
 use crate::{AsContextMut, StoreContextMut, Trap};
@@ -11,7 +13,7 @@ use core::fmt;
 use core::marker;
 use core::mem::MaybeUninit;
 use core::ptr::NonNull;
-use core::sync::atomic::{AtomicU64, Ordering::Relaxed};
+use core::sync::atomic::{AtomicU32, Ordering::Relaxed};
 use wasmtime_environ::component::{
     CanonicalAbiInfo, ComponentTypes, DefinedResourceIndex, InterfaceType, ResourceIndex,
     TypeResourceTableIndex,
@@ -248,12 +250,18 @@ pub struct Resource<T> {
 ///   borrow tracking associated with it. The low 32-bits of the value are
 ///   the table index and the upper 32-bits are the generation.
 ///
-/// Note that this is an `AtomicU64` but it's not intended to actually be
-/// used in conjunction with threads as generally a `Store<T>` lives on one
-/// thread at a time. The `AtomicU64` here is used to ensure that this type
-/// is `Send + Sync` when captured as a reference to make async programming
-/// more ergonomic.
-struct AtomicResourceState(AtomicU64);
+/// Note that this is two `AtomicU32` fields but it's not intended to actually
+/// be used in conjunction with threads as generally a `Store<T>` lives on one
+/// thread at a time. The pair of `AtomicU32` here is used to ensure that this
+/// type is `Send + Sync` when captured as a reference to make async
+/// programming more ergonomic.
+///
+/// Also note that two `AtomicU32` here are used instead of `AtomicU64` to be
+/// more portable to platforms without 64-bit atomics.
+struct AtomicResourceState {
+    index: AtomicU32,
+    generation: AtomicU32,
+}
 
 #[derive(Debug, PartialEq, Eq, Copy, Clone)]
 enum ResourceState {
@@ -264,39 +272,52 @@ enum ResourceState {
 }
 
 impl AtomicResourceState {
-    const BORROW: Self = Self(AtomicU64::new(ResourceState::BORROW));
-    const NOT_IN_TABLE: Self = Self(AtomicU64::new(ResourceState::NOT_IN_TABLE));
+    const BORROW: Self = AtomicResourceState::new(ResourceState::Borrow);
+    const NOT_IN_TABLE: Self = AtomicResourceState::new(ResourceState::NotInTable);
+
+    const fn new(state: ResourceState) -> AtomicResourceState {
+        let (index, generation) = state.encode();
+        Self {
+            index: AtomicU32::new(index),
+            generation: AtomicU32::new(generation),
+        }
+    }
 
     fn get(&self) -> ResourceState {
-        ResourceState::decode(self.0.load(Relaxed))
+        ResourceState::decode(self.index.load(Relaxed), self.generation.load(Relaxed))
     }
 
     fn swap(&self, state: ResourceState) -> ResourceState {
-        ResourceState::decode(self.0.swap(state.encode(), Relaxed))
+        let (index, generation) = state.encode();
+        let index_prev = self.index.load(Relaxed);
+        self.index.store(index, Relaxed);
+        let generation_prev = self.generation.load(Relaxed);
+        self.generation.store(generation, Relaxed);
+        ResourceState::decode(index_prev, generation_prev)
     }
 }
 
 impl ResourceState {
     // See comments on `state` above for info about these values.
-    const BORROW: u64 = u64::MAX;
-    const NOT_IN_TABLE: u64 = u64::MAX - 1;
-    const TAKEN: u64 = u64::MAX - 2;
+    const BORROW: u32 = u32::MAX;
+    const NOT_IN_TABLE: u32 = u32::MAX - 1;
+    const TAKEN: u32 = u32::MAX - 2;
 
-    fn decode(bits: u64) -> ResourceState {
-        match bits {
+    fn decode(idx: u32, generation: u32) -> ResourceState {
+        match generation {
             Self::BORROW => Self::Borrow,
             Self::NOT_IN_TABLE => Self::NotInTable,
             Self::TAKEN => Self::Taken,
-            other => Self::Index(HostResourceIndex(other)),
+            _ => Self::Index(HostResourceIndex::new(idx, generation)),
         }
     }
 
-    fn encode(&self) -> u64 {
+    const fn encode(&self) -> (u32, u32) {
         match self {
-            Self::Borrow => Self::BORROW,
-            Self::NotInTable => Self::NOT_IN_TABLE,
-            Self::Taken => Self::TAKEN,
-            Self::Index(index) => index.0,
+            Self::Borrow => (0, Self::BORROW),
+            Self::NotInTable => (0, Self::NOT_IN_TABLE),
+            Self::Taken => (0, Self::TAKEN),
+            Self::Index(index) => (index.index(), index.generation()),
         }
     }
 }
@@ -355,16 +376,16 @@ struct TableSlot {
 pub struct HostResourceIndex(u64);
 
 impl HostResourceIndex {
-    fn new(idx: u32, gen: u32) -> HostResourceIndex {
-        HostResourceIndex(u64::from(idx) | (u64::from(gen) << 32))
+    fn new(idx: u32, generation: u32) -> HostResourceIndex {
+        HostResourceIndex(u64::from(idx) | (u64::from(generation) << 32))
     }
 
-    fn index(&self) -> u32 {
-        u32::try_from(self.0 & 0xffffffff).unwrap()
+    const fn index(&self) -> u32 {
+        (self.0 & 0xffffffff) as u32
     }
 
-    fn gen(&self) -> u32 {
-        u32::try_from(self.0 >> 32).unwrap()
+    const fn generation(&self) -> u32 {
+        (self.0 >> 32) as u32
     }
 }
 
@@ -375,7 +396,7 @@ impl<'a> HostResourceTables<'a> {
             ResourceTables {
                 host_table: Some(host_table),
                 calls,
-                tables: None,
+                guest: None,
             },
             host_resource_data,
         )
@@ -400,13 +421,14 @@ impl<'a> HostResourceTables<'a> {
     /// if `idx` can't be lifted as an `own` (e.g. it has active borrows).
     pub fn host_resource_lift_own(&mut self, idx: HostResourceIndex) -> Result<u32> {
         let (idx, _) = self.validate_host_index(idx, true)?;
-        self.tables.resource_lift_own(None, idx)
+        self.tables.resource_lift_own(TypedResourceIndex::Host(idx))
     }
 
     /// See [`HostResourceTables::host_resource_lift_own`].
     pub fn host_resource_lift_borrow(&mut self, idx: HostResourceIndex) -> Result<u32> {
         let (idx, _) = self.validate_host_index(idx, false)?;
-        self.tables.resource_lift_borrow(None, idx)
+        self.tables
+            .resource_lift_borrow(TypedResourceIndex::Host(idx))
     }
 
     /// Lowers an `own` resource to be owned by the host.
@@ -423,13 +445,15 @@ impl<'a> HostResourceTables<'a> {
         dtor: Option<NonNull<VMFuncRef>>,
         flags: Option<InstanceFlags>,
     ) -> Result<HostResourceIndex> {
-        let idx = self.tables.resource_lower_own(None, rep)?;
+        let idx = self.tables.resource_lower_own(TypedResource::Host(rep))?;
         Ok(self.new_host_index(idx, dtor, flags))
     }
 
     /// See [`HostResourceTables::host_resource_lower_own`].
     pub fn host_resource_lower_borrow(&mut self, rep: u32) -> Result<HostResourceIndex> {
-        let idx = self.tables.resource_lower_borrow(None, rep)?;
+        let idx = self
+            .tables
+            .resource_lower_borrow(TypedResource::Host(rep))?;
         Ok(self.new_host_index(idx, None, None))
     }
 
@@ -453,7 +477,7 @@ impl<'a> HostResourceTables<'a> {
         // situation the operation that this is guarding will return a more
         // precise error, such as a lift operation.
         if let Some(actual) = actual {
-            if actual.generation != idx.gen() {
+            if actual.generation != idx.generation() {
                 bail!("host-owned resource is being used with the wrong type");
             }
         }
@@ -520,7 +544,7 @@ impl<'a> HostResourceTables<'a> {
     /// in the host tables.
     fn host_resource_drop(&mut self, idx: HostResourceIndex) -> Result<Option<(u32, TableSlot)>> {
         let (idx, slot) = self.validate_host_index(idx, true)?;
-        match self.tables.resource_drop(None, idx)? {
+        match self.tables.resource_drop(TypedResourceIndex::Host(idx))? {
             Some(rep) => Ok(Some((rep, slot.unwrap()))),
             None => Ok(None),
         }
@@ -535,7 +559,8 @@ impl<'a> HostResourceTables<'a> {
         rep: u32,
         ty: TypeResourceTableIndex,
     ) -> Result<u32> {
-        self.tables.resource_lower_own(Some(ty), rep)
+        self.tables
+            .resource_lower_own(TypedResource::Component { ty, rep })
     }
 
     /// Lowers a `borrow` resource into the guest, converting the `rep`
@@ -552,15 +577,21 @@ impl<'a> HostResourceTables<'a> {
         rep: u32,
         ty: TypeResourceTableIndex,
     ) -> Result<u32> {
-        self.tables.resource_lower_borrow(Some(ty), rep)
+        self.tables
+            .resource_lower_borrow(TypedResource::Component { ty, rep })
     }
 
     /// Lifts an `own` resource from the `idx` specified from the table `ty`.
     ///
     /// This will lookup the appropriate table in the guest and return the `rep`
     /// corresponding to `idx` if it's valid.
-    pub fn guest_resource_lift_own(&mut self, idx: u32, ty: TypeResourceTableIndex) -> Result<u32> {
-        self.tables.resource_lift_own(Some(ty), idx)
+    pub fn guest_resource_lift_own(
+        &mut self,
+        index: u32,
+        ty: TypeResourceTableIndex,
+    ) -> Result<u32> {
+        self.tables
+            .resource_lift_own(TypedResourceIndex::Component { ty, index })
     }
 
     /// Lifts a `borrow` resource from the `idx` specified from the table `ty`.
@@ -569,10 +600,11 @@ impl<'a> HostResourceTables<'a> {
     /// corresponding to `idx` if it's valid.
     pub fn guest_resource_lift_borrow(
         &mut self,
-        idx: u32,
+        index: u32,
         ty: TypeResourceTableIndex,
     ) -> Result<u32> {
-        self.tables.resource_lift_borrow(Some(ty), idx)
+        self.tables
+            .resource_lift_borrow(TypedResourceIndex::Component { ty, index })
     }
 
     /// Begins a call into the component instance, starting recording of
@@ -1034,7 +1066,7 @@ impl ResourceAny {
         // destructors have al been previously type-checked and are guaranteed
         // to take one i32 argument and return no results, so the parameters
         // here should be configured correctly.
-        unsafe { crate::Func::call_unchecked_raw(store, dtor, &mut args) }
+        unsafe { crate::Func::call_unchecked_raw(store, dtor, NonNull::from(&mut args)) }
     }
 
     fn lower_to_index<U>(&self, cx: &mut LowerContext<'_, U>, ty: InterfaceType) -> Result<u32> {
