@@ -7,6 +7,8 @@
 use crate::prelude::*;
 use crate::runtime::vm::vmcontext::{VMFuncRef, VMTableDefinition};
 use crate::runtime::vm::{GcStore, SendSyncPtr, VMGcRef, VMStore};
+use core::alloc::Layout;
+use core::mem;
 use core::ops::Range;
 use core::ptr::{self, NonNull};
 use core::slice;
@@ -263,9 +265,49 @@ fn wasm_to_table_type(ty: WasmRefType) -> TableElementType {
     match ty.heap_type.top() {
         WasmHeapTopType::Func => TableElementType::Func,
         WasmHeapTopType::Any | WasmHeapTopType::Extern => TableElementType::GcRef,
+        WasmHeapTopType::Cont => todo!(), // FIXME: #10248 stack switching support.
     }
 }
 
+/// Allocate dynamic table elements of the given length.
+///
+/// Relies on the fact that our tables' elements are initialized to `None`,
+/// which is represented by zero, to allocate pre-zeroed memory from the global
+/// allocator and avoid manual zero-initialization.
+///
+/// # Safety
+///
+/// Should only ever be called with a `T` that is a table element type and where
+/// `Option<T>`'s `None` variant is represented with zero.
+unsafe fn alloc_dynamic_table_elements<T>(len: usize) -> Result<Vec<Option<T>>> {
+    debug_assert!(
+        core::mem::MaybeUninit::<Option<T>>::zeroed()
+            .assume_init()
+            .is_none(),
+        "null table elements are represented with zeroed memory"
+    );
+
+    if len == 0 {
+        return Ok(vec![]);
+    }
+
+    let align = mem::align_of::<Option<T>>();
+
+    let size = mem::size_of::<Option<T>>();
+    let size = size.next_multiple_of(align);
+    let size = size.checked_mul(len).unwrap();
+
+    let layout = Layout::from_size_align(size, align)?;
+
+    let ptr = alloc::alloc::alloc_zeroed(layout);
+    ensure!(!ptr.is_null(), "failed to allocate memory for table");
+
+    let elems = Vec::<Option<T>>::from_raw_parts(ptr.cast(), len, len);
+    debug_assert!(elems.iter().all(|e| e.is_none()));
+
+    Ok(elems)
+}
+
 impl Table {
     /// Create a new dynamic (movable) table instance for the specified table plan.
     pub fn new_dynamic(
@@ -276,12 +318,12 @@ impl Table {
         let (minimum, maximum) = Self::limit_new(ty, store)?;
         match wasm_to_table_type(ty.ref_type) {
             TableElementType::Func => Ok(Self::from(DynamicFuncTable {
-                elements: vec![None; minimum],
+                elements: unsafe { alloc_dynamic_table_elements(minimum)? },
                 maximum,
                 lazy_init: tunables.table_lazy_init,
             })),
             TableElementType::GcRef => Ok(Self::from(DynamicGcRefTable {
-                elements: (0..minimum).map(|_| None).collect(),
+                elements: unsafe { alloc_dynamic_table_elements(minimum)? },
                 maximum,
             })),
         }
@@ -491,7 +533,7 @@ impl Table {
     /// `gc_store.is_none()` and this is a table of GC references.
     pub fn fill(
         &mut self,
-        gc_store: Option<&mut GcStore>,
+        mut gc_store: Option<&mut GcStore>,
         dst: u64,
         val: TableElement,
         len: u64,
@@ -512,18 +554,25 @@ impl Table {
                 funcrefs[start..end].fill(TaggedFuncRef::from(f, lazy_init));
             }
             TableElement::GcRef(r) => {
-                let gc_store =
-                    gc_store.expect("must provide a GcStore for tables of GC references");
-
                 // Clone the init GC reference into each table slot.
                 for slot in &mut self.gc_refs_mut()[start..end] {
-                    gc_store.write_gc_ref(slot, r.as_ref());
+                    match gc_store.as_deref_mut() {
+                        Some(s) => s.write_gc_ref(slot, r.as_ref()),
+                        None => {
+                            debug_assert!(slot.as_ref().is_none_or(|x| x.is_i31()));
+                            debug_assert!(r.as_ref().is_none_or(|r| r.is_i31()));
+                            *slot = r.as_ref().map(|r| r.copy_i31());
+                        }
+                    }
                 }
 
                 // Drop the init GC reference, since we aren't holding onto this
                 // reference anymore, only the clones in the table.
                 if let Some(r) = r {
-                    gc_store.drop_gc_ref(r);
+                    match gc_store {
+                        Some(s) => s.drop_gc_ref(r),
+                        None => debug_assert!(r.is_i31()),
+                    }
                 }
             }
             TableElement::UninitFunc => {
@@ -568,7 +617,7 @@ impl Table {
         if delta == 0 {
             return Ok(Some(old_size));
         }
-        // Cannot return `Trap::TableOutOfBounds` here becase `impl std::error::Error for Trap` is not available in no-std.
+        // Cannot return `Trap::TableOutOfBounds` here because `impl std::error::Error for Trap` is not available in no-std.
         let delta =
             usize::try_from(delta).map_err(|_| format_err!("delta exceeds host pointer size"))?;
 
@@ -627,7 +676,7 @@ impl Table {
         }
 
         self.fill(
-            store.store_opaque_mut().optional_gc_store_mut()?,
+            store.store_opaque_mut().optional_gc_store_mut(),
             u64::try_from(old_size).unwrap(),
             init_value,
             u64::try_from(delta).unwrap(),
@@ -653,7 +702,11 @@ impl Table {
                     .map(|e| e.into_table_element(lazy_init))
             }
             TableElementType::GcRef => self.gc_refs().get(index).map(|r| {
-                let r = r.as_ref().map(|r| gc_store.unwrap().clone_gc_ref(r));
+                let r = r.as_ref().map(|r| match gc_store {
+                    Some(s) => s.clone_gc_ref(r),
+                    None => r.copy_i31(),
+                });
+
                 TableElement::GcRef(r)
             }),
         }
@@ -740,25 +793,29 @@ impl Table {
         match self {
             Table::Static(StaticTable::Func(StaticFuncTable { data, size, .. })) => {
                 VMTableDefinition {
-                    base: data.as_ptr().cast(),
+                    base: data.cast().into(),
                     current_elements: *size,
                 }
             }
             Table::Static(StaticTable::GcRef(StaticGcRefTable { data, size })) => {
                 VMTableDefinition {
-                    base: data.as_ptr().cast(),
+                    base: data.cast().into(),
                     current_elements: *size,
                 }
             }
             Table::Dynamic(DynamicTable::Func(DynamicFuncTable { elements, .. })) => {
                 VMTableDefinition {
-                    base: elements.as_mut_ptr().cast(),
+                    base: NonNull::<[FuncTableElem]>::from(&mut elements[..])
+                        .cast()
+                        .into(),
                     current_elements: elements.len(),
                 }
             }
             Table::Dynamic(DynamicTable::GcRef(DynamicGcRefTable { elements, .. })) => {
                 VMTableDefinition {
-                    base: elements.as_mut_ptr().cast(),
+                    base: NonNull::<[Option<VMGcRef>]>::from(&mut elements[..])
+                        .cast()
+                        .into(),
                     current_elements: elements.len(),
                 }
             }
