@@ -20,6 +20,9 @@ const FEATURES: &[&str] = &[
     "CRANELIFT",
     "WINCH",
     "DEBUG_BUILTINS",
+    "WAT",
+    "POOLING_ALLOCATOR",
+    "COMPONENT_MODEL",
 ];
 // ... if you add a line above this be sure to change the other locations
 // marked WASMTIME_FEATURE_LIST
