@@ -1,8 +1,6 @@
 use crate::component::{AllCallFunc, ComponentTranslation, ComponentTypesBuilder, TrampolineIndex};
-use crate::prelude::*;
-use crate::Tunables;
+use crate::{CompiledFunctionBody, Tunables};
 use anyhow::Result;
-use std::any::Any;
 
 /// Compilation support necessary for components.
 pub trait ComponentCompiler: Send + Sync {
@@ -18,5 +16,5 @@ pub trait ComponentCompiler: Send + Sync {
         types: &ComponentTypesBuilder,
         trampoline: TrampolineIndex,
         tunables: &Tunables,
-    ) -> Result<AllCallFunc<Box<dyn Any + Send>>>;
+    ) -> Result<AllCallFunc<CompiledFunctionBody>>;
 }
