@@ -134,7 +134,9 @@ pub(super) fn run(
         inliner.record_export(name, def, types, &mut export_map)?;
     }
     inliner.result.exports = export_map;
-    inliner.result.num_resource_tables = types.num_resource_tables();
+    inliner.result.num_future_tables = types.num_future_tables();
+    inliner.result.num_stream_tables = types.num_stream_tables();
+    inliner.result.num_error_context_tables = types.num_error_context_tables();
 
     Ok(inliner.result)
 }
@@ -201,6 +203,7 @@ struct InlinerFrame<'a> {
     memories: PrimaryMap<MemoryIndex, dfg::CoreExport<EntityIndex>>,
     tables: PrimaryMap<TableIndex, dfg::CoreExport<EntityIndex>>,
     globals: PrimaryMap<GlobalIndex, dfg::CoreExport<EntityIndex>>,
+    tags: PrimaryMap<GlobalIndex, dfg::CoreExport<EntityIndex>>,
     modules: PrimaryMap<ModuleIndex, ModuleDef<'a>>,
 
     // component model index spaces
@@ -667,6 +670,326 @@ impl<'a> Inliner<'a> {
                     .push((*ty, dfg::Trampoline::ResourceDrop(id)));
                 frame.funcs.push(dfg::CoreDef::Trampoline(index));
             }
+            BackpressureSet { func } => {
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::BackpressureSet {
+                        instance: frame.instance,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            TaskReturn {
+                func,
+                result,
+                options,
+            } => {
+                let results = result
+                    .iter()
+                    .map(|ty| types.valtype(frame.translation.types_ref(), ty))
+                    .collect::<Result<_>>()?;
+                let results = types.new_tuple_type(results);
+                let options = self.adapter_options(frame, types, options);
+                let options = self.canonical_options(options);
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::TaskReturn { results, options }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            WaitableSetNew { func } => {
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::WaitableSetNew {
+                        instance: frame.instance,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            WaitableSetWait {
+                func,
+                async_,
+                memory,
+            } => {
+                let (memory, _) = self.memory(frame, types, *memory);
+                let memory = self.result.memories.push(memory);
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::WaitableSetWait {
+                        instance: frame.instance,
+                        async_: *async_,
+                        memory,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            WaitableSetPoll {
+                func,
+                async_,
+                memory,
+            } => {
+                let (memory, _) = self.memory(frame, types, *memory);
+                let memory = self.result.memories.push(memory);
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::WaitableSetPoll {
+                        instance: frame.instance,
+                        async_: *async_,
+                        memory,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            WaitableSetDrop { func } => {
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::WaitableSetDrop {
+                        instance: frame.instance,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            WaitableJoin { func } => {
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::WaitableJoin {
+                        instance: frame.instance,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            Yield { func, async_ } => {
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::Yield { async_: *async_ }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            SubtaskDrop { func } => {
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::SubtaskDrop {
+                        instance: frame.instance,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            StreamNew { ty, func } => {
+                let InterfaceType::Stream(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::StreamNew { ty }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            StreamRead { ty, func, options } => {
+                let InterfaceType::Stream(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let options = self.adapter_options(frame, types, options);
+                let options = self.canonical_options(options);
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::StreamRead { ty, options }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            StreamWrite { ty, func, options } => {
+                let InterfaceType::Stream(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let options = self.adapter_options(frame, types, options);
+                let options = self.canonical_options(options);
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::StreamWrite { ty, options }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            StreamCancelRead { ty, func, async_ } => {
+                let InterfaceType::Stream(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::StreamCancelRead {
+                        ty,
+                        async_: *async_,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            StreamCancelWrite { ty, func, async_ } => {
+                let InterfaceType::Stream(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::StreamCancelWrite {
+                        ty,
+                        async_: *async_,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            StreamCloseReadable { ty, func } => {
+                let InterfaceType::Stream(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::StreamCloseReadable { ty }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            StreamCloseWritable { ty, func } => {
+                let InterfaceType::Stream(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::StreamCloseWritable { ty }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            FutureNew { ty, func } => {
+                let InterfaceType::Future(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::FutureNew { ty }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            FutureRead { ty, func, options } => {
+                let InterfaceType::Future(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let options = self.adapter_options(frame, types, options);
+                let options = self.canonical_options(options);
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::FutureRead { ty, options }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            FutureWrite { ty, func, options } => {
+                let InterfaceType::Future(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let options = self.adapter_options(frame, types, options);
+                let options = self.canonical_options(options);
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::FutureWrite { ty, options }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            FutureCancelRead { ty, func, async_ } => {
+                let InterfaceType::Future(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::FutureCancelRead {
+                        ty,
+                        async_: *async_,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            FutureCancelWrite { ty, func, async_ } => {
+                let InterfaceType::Future(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::FutureCancelWrite {
+                        ty,
+                        async_: *async_,
+                    },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            FutureCloseReadable { ty, func } => {
+                let InterfaceType::Future(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::FutureCloseReadable { ty }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            FutureCloseWritable { ty, func } => {
+                let InterfaceType::Future(ty) =
+                    types.defined_type(frame.translation.types_ref(), *ty)?
+                else {
+                    unreachable!()
+                };
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::FutureCloseWritable { ty }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            ErrorContextNew { func, options } => {
+                let ty = types.error_context_table_type()?;
+                let options = self.adapter_options(frame, types, options);
+                let options = self.canonical_options(options);
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::ErrorContextNew { ty, options }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            ErrorContextDebugMessage { func, options } => {
+                let ty = types.error_context_table_type()?;
+                let options = self.adapter_options(frame, types, options);
+                let options = self.canonical_options(options);
+                let index = self.result.trampolines.push((
+                    *func,
+                    dfg::Trampoline::ErrorContextDebugMessage { ty, options },
+                ));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
+            ErrorContextDrop { func } => {
+                let ty = types.error_context_table_type()?;
+                let index = self
+                    .result
+                    .trampolines
+                    .push((*func, dfg::Trampoline::ErrorContextDrop { ty }));
+                frame.funcs.push(dfg::CoreDef::Trampoline(index));
+            }
 
             ModuleStatic(idx, ty) => {
                 frame.modules.push(ModuleDef::Static(*idx, *ty));
@@ -829,6 +1152,15 @@ impl<'a> Inliner<'a> {
                 );
             }
 
+            AliasExportTag(instance, name) => {
+                frame.tags.push(
+                    match self.core_def_of_module_instance_export(frame, *instance, *name) {
+                        dfg::CoreDef::Export(e) => e,
+                        _ => unreachable!(),
+                    },
+                );
+            }
+
             AliasComponentExport(instance, name) => {
                 match &frame.component_instances[*instance] {
                     // Aliasing an export from an imported instance means that
@@ -944,10 +1276,46 @@ impl<'a> Inliner<'a> {
                 EntityIndex::Table(i) => frame.tables[i].clone().into(),
                 EntityIndex::Global(i) => frame.globals[i].clone().into(),
                 EntityIndex::Memory(i) => frame.memories[i].clone().into(),
+                EntityIndex::Tag(_) => todo!(), // FIXME: #10252 support for tags in the component model
             },
         }
     }
 
+    fn memory(
+        &mut self,
+        frame: &InlinerFrame<'a>,
+        types: &ComponentTypesBuilder,
+        memory: MemoryIndex,
+    ) -> (dfg::CoreExport<MemoryIndex>, bool) {
+        let memory = frame.memories[memory].clone().map_index(|i| match i {
+            EntityIndex::Memory(i) => i,
+            _ => unreachable!(),
+        });
+        let memory64 = match &self.runtime_instances[memory.instance] {
+            InstanceModule::Static(idx) => match &memory.item {
+                ExportItem::Index(i) => {
+                    let ty = &self.nested_modules[*idx].module.memories[*i];
+                    match ty.idx_type {
+                        IndexType::I32 => false,
+                        IndexType::I64 => true,
+                    }
+                }
+                ExportItem::Name(_) => unreachable!(),
+            },
+            InstanceModule::Import(ty) => match &memory.item {
+                ExportItem::Name(name) => match types[*ty].exports[name] {
+                    EntityType::Memory(m) => match m.idx_type {
+                        IndexType::I32 => false,
+                        IndexType::I64 => true,
+                    },
+                    _ => unreachable!(),
+                },
+                ExportItem::Index(_) => unreachable!(),
+            },
+        };
+        (memory, memory64)
+    }
+
     /// Translates a `LocalCanonicalOptions` which indexes into the `frame`
     /// specified into a runtime representation.
     fn adapter_options(
@@ -956,38 +1324,15 @@ impl<'a> Inliner<'a> {
         types: &ComponentTypesBuilder,
         options: &LocalCanonicalOptions,
     ) -> AdapterOptions {
-        let memory = options.memory.map(|i| {
-            frame.memories[i].clone().map_index(|i| match i {
-                EntityIndex::Memory(i) => i,
-                _ => unreachable!(),
+        let (memory, memory64) = options
+            .memory
+            .map(|i| {
+                let (memory, memory64) = self.memory(frame, types, i);
+                (Some(memory), memory64)
             })
-        });
-        let memory64 = match &memory {
-            Some(memory) => match &self.runtime_instances[memory.instance] {
-                InstanceModule::Static(idx) => match &memory.item {
-                    ExportItem::Index(i) => {
-                        let ty = &self.nested_modules[*idx].module.memories[*i];
-                        match ty.idx_type {
-                            IndexType::I32 => false,
-                            IndexType::I64 => true,
-                        }
-                    }
-                    ExportItem::Name(_) => unreachable!(),
-                },
-                InstanceModule::Import(ty) => match &memory.item {
-                    ExportItem::Name(name) => match types[*ty].exports[name] {
-                        EntityType::Memory(m) => match m.idx_type {
-                            IndexType::I32 => false,
-                            IndexType::I64 => true,
-                        },
-                        _ => unreachable!(),
-                    },
-                    ExportItem::Index(_) => unreachable!(),
-                },
-            },
-            None => false,
-        };
+            .unwrap_or((None, false));
         let realloc = options.realloc.map(|i| frame.funcs[i].clone());
+        let callback = options.callback.map(|i| frame.funcs[i].clone());
         let post_return = options.post_return.map(|i| frame.funcs[i].clone());
         AdapterOptions {
             instance: frame.instance,
@@ -995,7 +1340,9 @@ impl<'a> Inliner<'a> {
             memory,
             memory64,
             realloc,
+            callback,
             post_return,
+            async_: options.async_,
         }
     }
 
@@ -1008,6 +1355,7 @@ impl<'a> Inliner<'a> {
             .memory
             .map(|export| self.result.memories.push(export));
         let realloc = options.realloc.map(|def| self.result.reallocs.push(def));
+        let callback = options.callback.map(|def| self.result.callbacks.push(def));
         let post_return = options
             .post_return
             .map(|def| self.result.post_returns.push(def));
@@ -1016,7 +1364,9 @@ impl<'a> Inliner<'a> {
             string_encoding: options.string_encoding,
             memory,
             realloc,
+            callback,
             post_return,
+            async_: options.async_,
         }
     }
 
@@ -1127,6 +1477,7 @@ impl<'a> InlinerFrame<'a> {
             memories: Default::default(),
             tables: Default::default(),
             globals: Default::default(),
+            tags: Default::default(),
 
             component_instances: Default::default(),
             component_funcs: Default::default(),
