@@ -198,8 +198,8 @@ impl WasmValType {
     /// Is this a type that is represented as a `VMGcRef` and is additionally
     /// not an `i31`?
     ///
-    /// That is, is this a a type that actually refers to an object allocated in
-    /// a GC heap?
+    /// That is, is this a type that actually refers to an object allocated in a
+    /// GC heap?
     #[inline]
     pub fn is_vmgcref_type_and_not_i31(&self) -> bool {
         match self {
@@ -221,6 +221,17 @@ impl WasmValType {
             | WasmValType::V128 => *self,
         }
     }
+
+    /// Attempt to build a `WasmValType` with the passed number of bits.
+    ///
+    /// Panics if the number of bits doesn't map to a WASM int type.
+    pub fn int_from_bits(bits: u8) -> Self {
+        match bits {
+            32 => Self::I32,
+            64 => Self::I64,
+            size => panic!("invalid int bits for WasmValType: {size}"),
+        }
+    }
 }
 
 /// WebAssembly reference type -- equivalent of `wasmparser`'s RefType
@@ -269,8 +280,8 @@ impl WasmRefType {
     /// Is this a type that is represented as a `VMGcRef` and is additionally
     /// not an `i31`?
     ///
-    /// That is, is this a a type that actually refers to an object allocated in
-    /// a GC heap?
+    /// That is, is this a type that actually refers to an object allocated in a
+    /// GC heap?
     #[inline]
     pub fn is_vmgcref_type_and_not_i31(&self) -> bool {
         self.heap_type.is_vmgcref_type_and_not_i31()
@@ -359,9 +370,12 @@ impl EngineOrModuleTypeIndex {
     }
 
     /// Get the underlying engine-level type index, or panic.
+    #[track_caller]
     pub fn unwrap_engine_type_index(self) -> VMSharedTypeIndex {
-        self.as_engine_type_index()
-            .unwrap_or_else(|| panic!("`unwrap_engine_type_index` on {self:?}"))
+        match self.as_engine_type_index() {
+            Some(x) => x,
+            None => panic!("`unwrap_engine_type_index` on {self:?}"),
+        }
     }
 
     /// Is this an module-level type index?
@@ -378,9 +392,12 @@ impl EngineOrModuleTypeIndex {
     }
 
     /// Get the underlying module-level type index, or panic.
+    #[track_caller]
     pub fn unwrap_module_type_index(self) -> ModuleInternedTypeIndex {
-        self.as_module_type_index()
-            .unwrap_or_else(|| panic!("`unwrap_module_type_index` on {self:?}"))
+        match self.as_module_type_index() {
+            Some(x) => x,
+            None => panic!("`unwrap_module_type_index` on {self:?}"),
+        }
     }
 
     /// Is this an recgroup-level type index?
@@ -397,15 +414,18 @@ impl EngineOrModuleTypeIndex {
     }
 
     /// Get the underlying module-level type index, or panic.
+    #[track_caller]
     pub fn unwrap_rec_group_type_index(self) -> RecGroupRelativeTypeIndex {
-        self.as_rec_group_type_index()
-            .unwrap_or_else(|| panic!("`unwrap_rec_group_type_index` on {self:?}"))
+        match self.as_rec_group_type_index() {
+            Some(x) => x,
+            None => panic!("`unwrap_rec_group_type_index` on {self:?}"),
+        }
     }
 }
 
 /// WebAssembly heap type -- equivalent of `wasmparser`'s HeapType
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
-#[allow(missing_docs, reason = "self-describing variants")]
+#[expect(missing_docs, reason = "self-describing variants")]
 pub enum WasmHeapType {
     // External types.
     Extern,
@@ -416,6 +436,11 @@ pub enum WasmHeapType {
     ConcreteFunc(EngineOrModuleTypeIndex),
     NoFunc,
 
+    // Continuation types.
+    Cont,
+    ConcreteCont(EngineOrModuleTypeIndex),
+    NoCont,
+
     // Internal types.
     Any,
     Eq,
@@ -434,6 +459,7 @@ impl From<WasmHeapTopType> for WasmHeapType {
             WasmHeapTopType::Extern => Self::Extern,
             WasmHeapTopType::Any => Self::Any,
             WasmHeapTopType::Func => Self::Func,
+            WasmHeapTopType::Cont => Self::Cont,
         }
     }
 }
@@ -445,6 +471,7 @@ impl From<WasmHeapBottomType> for WasmHeapType {
             WasmHeapBottomType::NoExtern => Self::NoExtern,
             WasmHeapBottomType::None => Self::None,
             WasmHeapBottomType::NoFunc => Self::NoFunc,
+            WasmHeapBottomType::NoCont => Self::NoCont,
         }
     }
 }
@@ -457,6 +484,9 @@ impl fmt::Display for WasmHeapType {
             Self::Func => write!(f, "func"),
             Self::ConcreteFunc(i) => write!(f, "func {i}"),
             Self::NoFunc => write!(f, "nofunc"),
+            Self::Cont => write!(f, "cont"),
+            Self::ConcreteCont(i) => write!(f, "cont {i}"),
+            Self::NoCont => write!(f, "nocont"),
             Self::Any => write!(f, "any"),
             Self::Eq => write!(f, "eq"),
             Self::I31 => write!(f, "i31"),
@@ -478,6 +508,7 @@ impl TypeTrace for WasmHeapType {
             Self::ConcreteArray(i) => func(i),
             Self::ConcreteFunc(i) => func(i),
             Self::ConcreteStruct(i) => func(i),
+            Self::ConcreteCont(i) => func(i),
             _ => Ok(()),
         }
     }
@@ -490,6 +521,7 @@ impl TypeTrace for WasmHeapType {
             Self::ConcreteArray(i) => func(i),
             Self::ConcreteFunc(i) => func(i),
             Self::ConcreteStruct(i) => func(i),
+            Self::ConcreteCont(i) => func(i),
             _ => Ok(()),
         }
     }
@@ -506,14 +538,15 @@ impl WasmHeapType {
 
             // All `t <: (ref null func)` are not.
             WasmHeapTopType::Func => false,
+            WasmHeapTopType::Cont => false,
         }
     }
 
     /// Is this a type that is represented as a `VMGcRef` and is additionally
     /// not an `i31`?
     ///
-    /// That is, is this a a type that actually refers to an object allocated in
-    /// a GC heap?
+    /// That is, is this a type that actually refers to an object allocated in a
+    /// GC heap?
     #[inline]
     pub fn is_vmgcref_type_and_not_i31(&self) -> bool {
         self.is_vmgcref_type() && *self != Self::I31
@@ -535,6 +568,10 @@ impl WasmHeapType {
                 WasmHeapTopType::Func
             }
 
+            WasmHeapType::Cont | WasmHeapType::ConcreteCont(_) | WasmHeapType::NoCont => {
+                WasmHeapTopType::Cont
+            }
+
             WasmHeapType::Any
             | WasmHeapType::Eq
             | WasmHeapType::I31
@@ -562,6 +599,10 @@ impl WasmHeapType {
                 WasmHeapBottomType::NoFunc
             }
 
+            WasmHeapType::Cont | WasmHeapType::ConcreteCont(_) | WasmHeapType::NoCont => {
+                WasmHeapBottomType::NoCont
+            }
+
             WasmHeapType::Any
             | WasmHeapType::Eq
             | WasmHeapType::I31
@@ -583,6 +624,8 @@ pub enum WasmHeapTopType {
     Any,
     /// The common supertype of all function references.
     Func,
+    /// The common supertype of all continuation references.
+    Cont,
 }
 
 /// A bottom heap type.
@@ -594,6 +637,8 @@ pub enum WasmHeapBottomType {
     None,
     /// The common subtype of all function references.
     NoFunc,
+    /// The common subtype of all continuation references.
+    NoCont,
 }
 
 /// WebAssembly function type -- equivalent of `wasmparser`'s FuncType.
@@ -741,6 +786,39 @@ impl WasmFuncType {
     }
 }
 
+/// WebAssembly continuation type -- equivalent of `wasmparser`'s ContType.
+#[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
+pub struct WasmContType(EngineOrModuleTypeIndex);
+
+impl fmt::Display for WasmContType {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "(cont {})", self.0)
+    }
+}
+
+impl WasmContType {
+    /// Constructs a new continuation type.
+    pub fn new(idx: EngineOrModuleTypeIndex) -> Self {
+        WasmContType(idx)
+    }
+}
+
+impl TypeTrace for WasmContType {
+    fn trace<F, E>(&self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        func(self.0)
+    }
+
+    fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        func(&mut self.0)
+    }
+}
+
 /// Represents storage types introduced in the GC spec for array and struct fields.
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
 pub enum WasmStorageType {
@@ -784,6 +862,20 @@ impl TypeTrace for WasmStorageType {
     }
 }
 
+impl WasmStorageType {
+    /// Is this a type that is represented as a `VMGcRef` and is additionally
+    /// not an `i31`?
+    ///
+    /// That is, is this a type that actually refers to an object allocated in a
+    /// GC heap?
+    pub fn is_vmgcref_type_and_not_i31(&self) -> bool {
+        match self {
+            WasmStorageType::I8 | WasmStorageType::I16 => false,
+            WasmStorageType::Val(v) => v.is_vmgcref_type_and_not_i31(),
+        }
+    }
+}
+
 /// The type of a struct field or array element.
 #[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
 pub struct WasmFieldType {
@@ -886,7 +978,7 @@ impl TypeTrace for WasmStructType {
 }
 
 #[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
-#[allow(missing_docs, reason = "self-describing type")]
+#[expect(missing_docs, reason = "self-describing type")]
 pub struct WasmCompositeType {
     /// The type defined inside the composite type.
     pub inner: WasmCompositeInnerType,
@@ -910,11 +1002,12 @@ impl fmt::Display for WasmCompositeType {
 
 /// A function, array, or struct type.
 #[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
-#[allow(missing_docs, reason = "self-describing variants")]
+#[expect(missing_docs, reason = "self-describing variants")]
 pub enum WasmCompositeInnerType {
     Array(WasmArrayType),
     Func(WasmFuncType),
     Struct(WasmStructType),
+    Cont(WasmContType),
 }
 
 impl fmt::Display for WasmCompositeInnerType {
@@ -923,11 +1016,12 @@ impl fmt::Display for WasmCompositeInnerType {
             Self::Array(ty) => fmt::Display::fmt(ty, f),
             Self::Func(ty) => fmt::Display::fmt(ty, f),
             Self::Struct(ty) => fmt::Display::fmt(ty, f),
+            Self::Cont(ty) => fmt::Display::fmt(ty, f),
         }
     }
 }
 
-#[allow(missing_docs, reason = "self-describing functions")]
+#[expect(missing_docs, reason = "self-describing functions")]
 impl WasmCompositeInnerType {
     #[inline]
     pub fn is_array(&self) -> bool {
@@ -982,6 +1076,24 @@ impl WasmCompositeInnerType {
     pub fn unwrap_struct(&self) -> &WasmStructType {
         self.as_struct().unwrap()
     }
+
+    #[inline]
+    pub fn is_cont(&self) -> bool {
+        matches!(self, Self::Cont(_))
+    }
+
+    #[inline]
+    pub fn as_cont(&self) -> Option<&WasmContType> {
+        match self {
+            Self::Cont(f) => Some(f),
+            _ => None,
+        }
+    }
+
+    #[inline]
+    pub fn unwrap_cont(&self) -> &WasmContType {
+        self.as_cont().unwrap()
+    }
 }
 
 impl TypeTrace for WasmCompositeType {
@@ -993,6 +1105,7 @@ impl TypeTrace for WasmCompositeType {
             WasmCompositeInnerType::Array(a) => a.trace(func),
             WasmCompositeInnerType::Func(f) => f.trace(func),
             WasmCompositeInnerType::Struct(a) => a.trace(func),
+            WasmCompositeInnerType::Cont(c) => c.trace(func),
         }
     }
 
@@ -1004,6 +1117,7 @@ impl TypeTrace for WasmCompositeType {
             WasmCompositeInnerType::Array(a) => a.trace_mut(func),
             WasmCompositeInnerType::Func(f) => f.trace_mut(func),
             WasmCompositeInnerType::Struct(a) => a.trace_mut(func),
+            WasmCompositeInnerType::Cont(c) => c.trace_mut(func),
         }
     }
 }
@@ -1042,7 +1156,7 @@ impl fmt::Display for WasmSubType {
 /// Implicitly define all of these helper functions to handle only unshared
 /// types; essentially, these act like `is_unshared_*` functions until shared
 /// support is implemented.
-#[allow(missing_docs, reason = "self-describing functions")]
+#[expect(missing_docs, reason = "self-describing functions")]
 impl WasmSubType {
     #[inline]
     pub fn is_func(&self) -> bool {
@@ -1103,6 +1217,26 @@ impl WasmSubType {
         assert!(!self.composite_type.shared);
         self.composite_type.inner.unwrap_struct()
     }
+
+    #[inline]
+    pub fn is_cont(&self) -> bool {
+        self.composite_type.inner.is_cont() && !self.composite_type.shared
+    }
+
+    #[inline]
+    pub fn as_cont(&self) -> Option<&WasmContType> {
+        if self.composite_type.shared {
+            None
+        } else {
+            self.composite_type.inner.as_cont()
+        }
+    }
+
+    #[inline]
+    pub fn unwrap_cont(&self) -> &WasmContType {
+        assert!(!self.composite_type.shared);
+        self.composite_type.inner.unwrap_cont()
+    }
 }
 
 impl TypeTrace for WasmSubType {
@@ -1295,6 +1429,11 @@ entity_impl!(DataIndex);
 pub struct ElemIndex(u32);
 entity_impl!(ElemIndex);
 
+/// Index type of a defined tag inside the WebAssembly module.
+#[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug, Serialize, Deserialize)]
+pub struct DefinedTagIndex(u32);
+entity_impl!(DefinedTagIndex);
+
 /// Index type of an event inside the WebAssembly module.
 #[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug, Serialize, Deserialize)]
 pub struct TagIndex(u32);
@@ -1319,6 +1458,8 @@ pub enum EntityIndex {
     Memory(MemoryIndex),
     /// Global index.
     Global(GlobalIndex),
+    /// Tag index.
+    Tag(TagIndex),
 }
 
 impl From<FuncIndex> for EntityIndex {
@@ -1345,6 +1486,12 @@ impl From<GlobalIndex> for EntityIndex {
     }
 }
 
+impl From<TagIndex> for EntityIndex {
+    fn from(idx: TagIndex) -> EntityIndex {
+        EntityIndex::Tag(idx)
+    }
+}
+
 /// A type of an item in a wasm module where an item is typically something that
 /// can be exported.
 #[derive(Clone, Debug, Serialize, Deserialize)]
@@ -1353,7 +1500,7 @@ pub enum EntityType {
     Global(Global),
     /// A linear memory with the specified limits
     Memory(Memory),
-    /// An event definition.
+    /// An exception and control tag definition.
     Tag(Tag),
     /// A table with the specified element type and limits
     Table(Table),
@@ -1371,7 +1518,8 @@ impl TypeTrace for EntityType {
             Self::Global(g) => g.trace(func),
             Self::Table(t) => t.trace(func),
             Self::Function(idx) => func(*idx),
-            Self::Memory(_) | Self::Tag(_) => Ok(()),
+            Self::Memory(_) => Ok(()),
+            Self::Tag(t) => t.trace(func),
         }
     }
 
@@ -1383,7 +1531,8 @@ impl TypeTrace for EntityType {
             Self::Global(g) => g.trace_mut(func),
             Self::Table(t) => t.trace_mut(func),
             Self::Function(idx) => func(idx),
-            Self::Memory(_) | Self::Tag(_) => Ok(()),
+            Self::Memory(_) => Ok(()),
+            Self::Tag(t) => t.trace_mut(func),
         }
     }
 }
@@ -1559,7 +1708,7 @@ impl ConstExpr {
 }
 
 /// The subset of Wasm opcodes that are constant.
-#[allow(missing_docs, reason = "self-describing variants")]
+#[expect(missing_docs, reason = "self-describing variants")]
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash, Serialize, Deserialize)]
 pub enum ConstOp {
     I32Const(i32),
@@ -1593,6 +1742,8 @@ pub enum ConstOp {
         array_type_index: TypeIndex,
         array_size: u32,
     },
+    ExternConvertAny,
+    AnyConvertExtern,
 }
 
 impl ConstOp {
@@ -1634,6 +1785,8 @@ impl ConstOp {
                 array_type_index: TypeIndex::from_u32(array_type_index),
                 array_size,
             },
+            O::ExternConvertAny => Self::ExternConvertAny,
+            O::AnyConvertExtern => Self::AnyConvertExtern,
             op => {
                 return Err(wasm_unsupported!(
                     "unsupported opcode in const expression at offset {offset:#x}: {op:?}",
@@ -1645,7 +1798,7 @@ impl ConstOp {
 
 /// The type that can be used to index into [Memory] and [Table].
 #[derive(Debug, Clone, Copy, Hash, Eq, PartialEq, Serialize, Deserialize)]
-#[allow(missing_docs, reason = "self-describing variants")]
+#[expect(missing_docs, reason = "self-describing variants")]
 pub enum IndexType {
     I32,
     I64,
@@ -1653,7 +1806,7 @@ pub enum IndexType {
 
 /// The size range of resizeable storage associated with [Memory] types and [Table] types.
 #[derive(Debug, Clone, Copy, Hash, Eq, PartialEq, Serialize, Deserialize)]
-#[allow(missing_docs, reason = "self-describing fields")]
+#[expect(missing_docs, reason = "self-describing fields")]
 pub struct Limits {
     pub min: u64,
     pub max: Option<u64>,
@@ -1870,6 +2023,12 @@ impl Memory {
             return false;
         }
 
+        // If its minimum and maximum are the same, then the memory will never
+        // be resized, and therefore will never move.
+        if self.limits.max.is_some_and(|max| self.limits.min == max) {
+            return false;
+        }
+
         // If the maximum size of this memory is above the threshold of the
         // initial memory reservation then the memory may move.
         let max = self.maximum_byte_size().unwrap_or(u64::MAX);
@@ -1878,7 +2037,7 @@ impl Memory {
 }
 
 #[derive(Copy, Clone, Debug)]
-#[allow(missing_docs, reason = "self-describing error struct")]
+#[expect(missing_docs, reason = "self-describing error struct")]
 pub struct SizeOverflow;
 
 impl fmt::Display for SizeOverflow {
@@ -1913,32 +2072,38 @@ impl From<wasmparser::MemoryType> for Memory {
     }
 }
 
-/// WebAssembly event.
+/// WebAssembly exception and control tag.
 #[derive(Debug, Clone, Copy, Hash, Eq, PartialEq, Serialize, Deserialize)]
 pub struct Tag {
-    /// The event signature type.
-    pub ty: TypeIndex,
+    /// The tag signature type.
+    pub signature: EngineOrModuleTypeIndex,
 }
 
-impl From<wasmparser::TagType> for Tag {
-    fn from(ty: wasmparser::TagType) -> Tag {
-        match ty.kind {
-            wasmparser::TagKind::Exception => Tag {
-                ty: TypeIndex::from_u32(ty.func_type_idx),
-            },
-        }
+impl TypeTrace for Tag {
+    fn trace<F, E>(&self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        func(self.signature)
+    }
+
+    fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        func(&mut self.signature)
     }
 }
 
 /// Helpers used to convert a `wasmparser` type to a type in this crate.
-#[allow(missing_docs, reason = "self-describing functions")]
+#[expect(missing_docs, reason = "self-describing functions")]
 pub trait TypeConvert {
     /// Converts a wasmparser table type into a wasmtime type
-    fn convert_global_type(&self, ty: &wasmparser::GlobalType) -> Global {
-        Global {
-            wasm_ty: self.convert_valtype(ty.content_type),
+    fn convert_global_type(&self, ty: &wasmparser::GlobalType) -> WasmResult<Global> {
+        Ok(Global {
+            wasm_ty: self.convert_valtype(ty.content_type)?,
             mutability: ty.mutable,
-        }
+        })
     }
 
     /// Converts a wasmparser table type into a wasmtime type
@@ -1954,106 +2119,118 @@ pub trait TypeConvert {
         Ok(Table {
             idx_type,
             limits,
-            ref_type: self.convert_ref_type(ty.element_type),
+            ref_type: self.convert_ref_type(ty.element_type)?,
         })
     }
 
-    fn convert_sub_type(&self, ty: &wasmparser::SubType) -> WasmSubType {
-        WasmSubType {
+    fn convert_sub_type(&self, ty: &wasmparser::SubType) -> WasmResult<WasmSubType> {
+        Ok(WasmSubType {
             is_final: ty.is_final,
             supertype: ty.supertype_idx.map(|i| self.lookup_type_index(i.unpack())),
-            composite_type: self.convert_composite_type(&ty.composite_type),
-        }
+            composite_type: self.convert_composite_type(&ty.composite_type)?,
+        })
     }
 
-    fn convert_composite_type(&self, ty: &wasmparser::CompositeType) -> WasmCompositeType {
+    fn convert_composite_type(
+        &self,
+        ty: &wasmparser::CompositeType,
+    ) -> WasmResult<WasmCompositeType> {
         let inner = match &ty.inner {
             wasmparser::CompositeInnerType::Func(f) => {
-                WasmCompositeInnerType::Func(self.convert_func_type(f))
+                WasmCompositeInnerType::Func(self.convert_func_type(f)?)
             }
             wasmparser::CompositeInnerType::Array(a) => {
-                WasmCompositeInnerType::Array(self.convert_array_type(a))
+                WasmCompositeInnerType::Array(self.convert_array_type(a)?)
             }
             wasmparser::CompositeInnerType::Struct(s) => {
-                WasmCompositeInnerType::Struct(self.convert_struct_type(s))
+                WasmCompositeInnerType::Struct(self.convert_struct_type(s)?)
             }
-            wasmparser::CompositeInnerType::Cont(_) => {
-                unimplemented!("continuation types")
+            wasmparser::CompositeInnerType::Cont(c) => {
+                WasmCompositeInnerType::Cont(self.convert_cont_type(c))
             }
         };
-        WasmCompositeType {
+        Ok(WasmCompositeType {
             inner,
             shared: ty.shared,
+        })
+    }
+
+    /// Converts a wasmparser continuation type to a wasmtime type
+    fn convert_cont_type(&self, ty: &wasmparser::ContType) -> WasmContType {
+        if let WasmHeapType::ConcreteFunc(sigidx) = self.lookup_heap_type(ty.0.unpack()) {
+            WasmContType::new(sigidx)
+        } else {
+            panic!("Failed to extract signature index for continuation type.")
         }
     }
 
-    fn convert_struct_type(&self, ty: &wasmparser::StructType) -> WasmStructType {
-        WasmStructType {
+    fn convert_struct_type(&self, ty: &wasmparser::StructType) -> WasmResult<WasmStructType> {
+        Ok(WasmStructType {
             fields: ty
                 .fields
                 .iter()
                 .map(|f| self.convert_field_type(f))
-                .collect(),
-        }
+                .collect::<WasmResult<_>>()?,
+        })
     }
 
-    fn convert_array_type(&self, ty: &wasmparser::ArrayType) -> WasmArrayType {
-        WasmArrayType(self.convert_field_type(&ty.0))
+    fn convert_array_type(&self, ty: &wasmparser::ArrayType) -> WasmResult<WasmArrayType> {
+        Ok(WasmArrayType(self.convert_field_type(&ty.0)?))
     }
 
-    fn convert_field_type(&self, ty: &wasmparser::FieldType) -> WasmFieldType {
-        WasmFieldType {
-            element_type: self.convert_storage_type(&ty.element_type),
+    fn convert_field_type(&self, ty: &wasmparser::FieldType) -> WasmResult<WasmFieldType> {
+        Ok(WasmFieldType {
+            element_type: self.convert_storage_type(&ty.element_type)?,
             mutable: ty.mutable,
-        }
+        })
     }
 
-    fn convert_storage_type(&self, ty: &wasmparser::StorageType) -> WasmStorageType {
-        match ty {
+    fn convert_storage_type(&self, ty: &wasmparser::StorageType) -> WasmResult<WasmStorageType> {
+        Ok(match ty {
             wasmparser::StorageType::I8 => WasmStorageType::I8,
             wasmparser::StorageType::I16 => WasmStorageType::I16,
-            wasmparser::StorageType::Val(v) => WasmStorageType::Val(self.convert_valtype(*v)),
-        }
+            wasmparser::StorageType::Val(v) => WasmStorageType::Val(self.convert_valtype(*v)?),
+        })
     }
 
     /// Converts a wasmparser function type to a wasmtime type
-    fn convert_func_type(&self, ty: &wasmparser::FuncType) -> WasmFuncType {
+    fn convert_func_type(&self, ty: &wasmparser::FuncType) -> WasmResult<WasmFuncType> {
         let params = ty
             .params()
             .iter()
             .map(|t| self.convert_valtype(*t))
-            .collect();
+            .collect::<WasmResult<_>>()?;
         let results = ty
             .results()
             .iter()
             .map(|t| self.convert_valtype(*t))
-            .collect();
-        WasmFuncType::new(params, results)
+            .collect::<WasmResult<_>>()?;
+        Ok(WasmFuncType::new(params, results))
     }
 
     /// Converts a wasmparser value type to a wasmtime type
-    fn convert_valtype(&self, ty: wasmparser::ValType) -> WasmValType {
-        match ty {
+    fn convert_valtype(&self, ty: wasmparser::ValType) -> WasmResult<WasmValType> {
+        Ok(match ty {
             wasmparser::ValType::I32 => WasmValType::I32,
             wasmparser::ValType::I64 => WasmValType::I64,
             wasmparser::ValType::F32 => WasmValType::F32,
             wasmparser::ValType::F64 => WasmValType::F64,
             wasmparser::ValType::V128 => WasmValType::V128,
-            wasmparser::ValType::Ref(t) => WasmValType::Ref(self.convert_ref_type(t)),
-        }
+            wasmparser::ValType::Ref(t) => WasmValType::Ref(self.convert_ref_type(t)?),
+        })
     }
 
     /// Converts a wasmparser reference type to a wasmtime type
-    fn convert_ref_type(&self, ty: wasmparser::RefType) -> WasmRefType {
-        WasmRefType {
+    fn convert_ref_type(&self, ty: wasmparser::RefType) -> WasmResult<WasmRefType> {
+        Ok(WasmRefType {
             nullable: ty.is_nullable(),
-            heap_type: self.convert_heap_type(ty.heap_type()),
-        }
+            heap_type: self.convert_heap_type(ty.heap_type())?,
+        })
     }
 
     /// Converts a wasmparser heap type to a wasmtime type
-    fn convert_heap_type(&self, ty: wasmparser::HeapType) -> WasmHeapType {
-        match ty {
+    fn convert_heap_type(&self, ty: wasmparser::HeapType) -> WasmResult<WasmHeapType> {
+        Ok(match ty {
             wasmparser::HeapType::Concrete(i) => self.lookup_heap_type(i),
             wasmparser::HeapType::Abstract { ty, shared: false } => match ty {
                 wasmparser::AbstractHeapType::Extern => WasmHeapType::Extern,
@@ -2071,11 +2248,11 @@ pub trait TypeConvert {
                 | wasmparser::AbstractHeapType::NoExn
                 | wasmparser::AbstractHeapType::Cont
                 | wasmparser::AbstractHeapType::NoCont => {
-                    unimplemented!("unsupported heap type {ty:?}");
+                    return Err(wasm_unsupported!("unsupported heap type {ty:?}"))
                 }
             },
-            _ => unimplemented!("unsupported heap type {ty:?}"),
-        }
+            _ => return Err(wasm_unsupported!("unsupported heap type {ty:?}")),
+        })
     }
 
     /// Converts the specified type index from a heap type into a canonicalized
