@@ -32,17 +32,19 @@
 //! The machine stack throughout the function call is as follows:
 //! ┌──────────────────────────────────────────────────┐
 //! │                                                  │
-//! │                  1                               │
 //! │  Stack space created by any previous spills      │
 //! │  from the value stack; and which memory values   │
 //! │  are used as function arguments.                 │
 //! │                                                  │
 //! ├──────────────────────────────────────────────────┤ ---> The Wasm value stack at this point in time would look like:
-//! │                                                  │      [ Mem(offset) | Mem(offset) | Local(index) | Local(index) ]
-//! │                   2                              │
+//! │                                                  │      
 //! │   Stack space created by spilling locals and     |
 //! │   registers at the callsite.                     │
 //! │                                                  │
+//! ├─────────────────────────────────────────────────┬┤
+//! │                                                  │
+//! │   Return Area (Multi-value results)              │
+//! │                                                  │
 //! │                                                  │
 //! ├─────────────────────────────────────────────────┬┤ ---> The Wasm value stack at this point in time would look like:
 //! │                                                  │      [ Mem(offset) | Mem(offset) | Mem(offset) | Mem(offset) ]
@@ -91,7 +93,7 @@ impl FnCall {
     ) -> Result<()> {
         let (kind, callee_context) = Self::lower(env, context.vmoffsets, &callee, context, masm)?;
 
-        let sig = env.callee_sig::<M::ABI>(&callee);
+        let sig = env.callee_sig::<M::ABI>(&callee)?;
         context.spill(masm)?;
         let ret_area = Self::make_ret_area(&sig, masm)?;
         let arg_stack_space = sig.params_stack_size();
@@ -142,11 +144,11 @@ impl FnCall {
         match callee {
             Callee::Builtin(b) => Ok(Self::lower_builtin(env, b)),
             Callee::FuncRef(_) => {
-                Self::lower_funcref(env.callee_sig::<M::ABI>(callee), ptr, context, masm)
+                Self::lower_funcref(env.callee_sig::<M::ABI>(callee)?, ptr, context, masm)
             }
             Callee::Local(i) => Ok(Self::lower_local(env, *i)),
             Callee::Import(i) => {
-                let sig = env.callee_sig::<M::ABI>(callee);
+                let sig = env.callee_sig::<M::ABI>(callee)?;
                 Self::lower_import(*i, sig, context, masm, vmoffsets)
             }
         }
@@ -163,7 +165,6 @@ impl FnCall {
                 CalleeKind::direct(env.name_builtin(idx)),
                 ContextArgs::pinned_vmctx(),
             ),
-            BuiltinType::LibCall(c) => (CalleeKind::libcall(c), ContextArgs::none()),
         }
     }
 
@@ -328,14 +329,14 @@ impl FnCall {
 
             match operand {
                 &ABIOperand::Reg { ty, reg, .. } => {
-                    masm.load_addr(addr, writable!(reg), ty.try_into()?)?;
+                    masm.compute_addr(addr, writable!(reg), ty.try_into()?)?;
                 }
                 &ABIOperand::Stack { ty, offset, .. } => {
                     let slot = masm.address_at_sp(SPOffset::from_u32(offset))?;
                     // Don't rely on `ABI::scratch_for` as we always use
                     // an int register as the return pointer.
                     let scratch = scratch!(M);
-                    masm.load_addr(addr, writable!(scratch), ty.try_into()?)?;
+                    masm.compute_addr(addr, writable!(scratch), ty.try_into()?)?;
                     masm.store(scratch.into(), slot, ty.try_into()?)?;
                 }
             }
