@@ -1,9 +1,8 @@
-use crate::Type;
+use crate::{Params, Type};
 use id_arena::{Arena, Id};
 use indexmap::IndexMap;
-use semver::Version;
 use serde::ser::{SerializeMap, SerializeSeq, Serializer};
-use serde::{de::Error, Deserialize, Serialize};
+use serde::Serialize;
 
 pub fn serialize_none<S>(serializer: S) -> Result<S::Ok, S::Error>
 where
@@ -68,17 +67,24 @@ impl Serialize for Type {
             Type::S16 => serializer.serialize_str("s16"),
             Type::S32 => serializer.serialize_str("s32"),
             Type::S64 => serializer.serialize_str("s64"),
-            Type::F32 => serializer.serialize_str("f32"),
-            Type::F64 => serializer.serialize_str("f64"),
+            Type::Float32 => serializer.serialize_str("float32"),
+            Type::Float64 => serializer.serialize_str("float64"),
             Type::Char => serializer.serialize_str("char"),
             Type::String => serializer.serialize_str("string"),
-            Type::ErrorContext => serializer.serialize_str("error-context"),
             Type::Id(type_id) => serializer.serialize_u64(type_id.index() as u64),
         }
     }
 }
 
-pub fn serialize_params<S>(params: &[(String, Type)], serializer: S) -> Result<S::Ok, S::Error>
+pub fn serialize_anon_result<S>(typ: &Type, serializer: S) -> Result<S::Ok, S::Error>
+where
+    S: Serializer,
+{
+    let params: Params = vec![(String::default(), *typ)];
+    serialize_params(&params, serializer)
+}
+
+pub fn serialize_params<S>(params: &Params, serializer: S) -> Result<S::Ok, S::Error>
 where
     S: Serializer,
 {
@@ -100,41 +106,3 @@ struct Param {
     #[serde(rename = "type")]
     pub typ: Type,
 }
-
-pub fn serialize_version<S>(version: &Version, serializer: S) -> Result<S::Ok, S::Error>
-where
-    S: Serializer,
-{
-    version.to_string().serialize(serializer)
-}
-
-pub fn deserialize_version<'de, D>(deserializer: D) -> Result<Version, D::Error>
-where
-    D: serde::de::Deserializer<'de>,
-{
-    let version: String = String::deserialize(deserializer)?;
-    version.parse().map_err(|e| D::Error::custom(e))
-}
-
-pub fn serialize_optional_version<S>(
-    version: &Option<Version>,
-    serializer: S,
-) -> Result<S::Ok, S::Error>
-where
-    S: Serializer,
-{
-    version
-        .as_ref()
-        .map(|s| s.to_string())
-        .serialize(serializer)
-}
-
-pub fn deserialize_optional_version<'de, D>(deserializer: D) -> Result<Option<Version>, D::Error>
-where
-    D: serde::de::Deserializer<'de>,
-{
-    match <Option<String>>::deserialize(deserializer)? {
-        Some(version) => Ok(Some(version.parse().map_err(|e| D::Error::custom(e))?)),
-        None => Ok(None),
-    }
-}
